/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.XExtensionManager;
import org.deckfour.xes.extension.std.XCostExtension;
import org.deckfour.xes.extension.std.XIdentityExtension;
import org.deckfour.xes.factory.XFactoryBufferedImpl;
import org.deckfour.xes.id.XID;
import org.deckfour.xes.in.XParser;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeID;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.util.XsDateTimeConversionJava7;

import com.opencsv.CSVReader;

/**
 * This class extends <code>XParser</code> to define a XES parser
 * for reading XES models from a given input stream in CSV format.
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 * 
 * @see XParser
 */
public class CsvXesParser extends XParser {
	
	/**
	 * Unique URI for the format definition.
	 */
	protected static final URI XES_URI = URI.create("http://www.xes-standard.org/");	
	
	/**
	 * Utility object for data conversion, from a date expressed in
	 * some natural form, to a date expressed in the XES standard.
	 * 
	 * @see XsDateTimeConversionJava7
	 */
	private XsDateTimeConversionJava7 xsDateTimeConverter = new XsDateTimeConversionJava7();
	
	/**
	 * Utility object for creating an XLog and its elements
	 * 
	 * @see XFactoryBufferedImpl
	 */
	private XFactoryBufferedImpl xesFactory = new XFactoryBufferedImpl();
	
	/**
	 * Variable for recording possible warnings that can occur during the parsing process
	 * 
	 * @see StringBuffer
	 */
	private StringBuffer warnings;
	
	/**
	 *  Variable for recording a possible error that may occur during the parsing process
	 */
	private String errors;
	
	/**
	 * Constructs a <code>CsvXesParser</code>, ready to use.
	 */
	public CsvXesParser () {
		warnings = new StringBuffer();
		errors = "";
	}

	/**
	 * Returns the name of the author of this parser.
	 */
	@Override
	public String author() {
		return "Francesco Santeramo";
	}

	/**
	 * Returns a brief description of this parser.
	 */
	@Override
	public String description() {
		return "Reads XES models from CSV serializations";
	}

	/**
	 * Returns the name of this parser or, more specifically, the name of the format it can process.
	 */
	@Override
	public String name() {
		return "CSV XES";
	}
	
	/**
	 * This method returns the possibile warning list recorded during the parsing process.
	 * 
	 * @return the <code>String</code> representing the warning list recorded during the parsing process.
	 */
	public String getWarnings() {
		return warnings.toString();
	}
	
	/**
	 * This method returns the possible error happened during the parsing process.
	 * 
	 * @return the <code>String</code> representing the error happened during the parsing process.
	 */
	public String getErrors() {
		return errors;
	}
	
	/**
	 * Checks whether this parser can handle the given file
	 * 
	 * @param file File to check against parser
	 * 
	 * @return Whether this parser can handle the given file.
	 */
	@Override
	public boolean canParse(File file) {
		String filename = file.getName();
		if (endsWithIgnoreCase(filename, ".csv") || endsWithIgnoreCase(filename, ".txt")) {
			return true;
		}
		return false;		
	}
	
	/**
	 * Parses the given file in a CSV format, and returns the XLog instances extracted. 
	 * 
	 * <p>The file is first checked against this parser, to check whether it can be handled. 
	 * 
	 * <p>If the parser cannot handle the given file, or the extraction itself fails, 
	 * the parser should raise an IOException.
	 * 
	 * @param file The file to be parsed
	 * 
	 * @return List of XLog instances parsed from the given file.
	 * 
	 * @throws Exception Raised in case the parser fails, or the given file cannot be processed.
	 */
	@Override
	public List<XLog> parse(File file) throws Exception { 
		InputStream is = new FileInputStream(file);
		return parse(is);
	}	

	/**
	 * Parses a log from the given input stream in a CSV format, and returns the XLog instances extracted.
	 * 
	 * @param is Stream to read XLog instances from
	 * 
	 * @return A list of XLog instances read from the given input stream. The number of read XLogs is at least one. If no XLog instance could be parsed, the parser is expected to throw an exception.
	 * 
	 * @throws Exception Raised in case the parser fails, or the given file cannot be processed.
	 */
	@Override
	public List<XLog> parse(InputStream is) throws Exception {
		// here we save the list of generated XLog
		ArrayList<XLog> logList = new ArrayList<XLog>();
		// open a csv reader over the stream, using try-with-resource statement
		try (BufferedReader fileReader = new BufferedReader(new InputStreamReader(is))) {				
			try (CSVReader reader = new CSVReader(fileReader)) {
				// read the stream and generate the XLog, saving it inside the list
				generateXLogs(reader, logList, 0);
				// if during the generation an error occurs
				if (errors.length() > 0) {
					// reset the XLog list
					logList = new ArrayList<XLog>();
				} else if (logList.isEmpty() && errors.equals("")) {
					// else if no error came up but no XLog gets generated
					errors = "The selected CSV file does not contain any valid text. Please select another file.";
				}
				// otherwise we have a list of XLog to return
			} catch(IOException e) {
			    errors = "Unable to read the CSV file. Please try again.";
			}
		} catch(IOException e) {
		    errors = "Unable to read the CSV file. Please try again.";
		}		
		// empty if an error occurred, with a valid log list otherwise
		return logList; 
	}
	
	/**
	 * This method first starts reading the file from top to bottom searching for the first line
	 * with some text. The first line of a CSV document is the column line, so when it finds it 
	 * (if it does not, it reports that the file is empty), it check the column names if they 
	 * represents valid XES attribute keys of XES extensions (if not, it reports the wrong name).
	 * 
	 * <p>After it has read the column line correctly and defined the column names, the method 
	 * reads all the remaining lines (every line of the CSV document represents an event and its 
	 * attributes), skipping the empty ones, taking the first line value as the trace id to which
	 * the event belongs and the remaining values of the line as the event values for the attributes
	 * defined by the column names.
	 * 
	 * <p>At the end of the process, the method adds the generated XLog to the collection and if 
	 * there is another log in the CSV to translate, it starts again to analyse this other log.
	 * 
	 * <p>The method manages empty values as the absence of that attribute for the scanned event 
	 * and discards the exceeding values (if a line has more values than the column names).
	 * 
	 * @param reader the opened <code>CSVReader</code> over the CSV documentto use for reading the file
	 * @param logList the <code>List</code> of <code>XLog</code> where to add the generated log
	 * @param line the line from which to start analysing (the first time this value is 0, the top of the document,
	 * 				then it assumes the values of the line where a new log in the CSV document starts)
	 * @throws Exception possible exceptions thrown by the CSVReader
	 */
	private void generateXLogs(CSVReader reader, List<XLog> logList, int line) throws Exception {		
		try {
			// get the line number in the document where to start
			int lineNumber = line;
			// read the first line, the column names
			String [] columns = reader.readNext();	
			lineNumber++;
			
			boolean columnFound = false;
			// if the read line is empty
			if (columns[0].trim().isEmpty()) {
				// continue scrolling the document, searching for a non empty line
				while ((columns = reader.readNext()) != null) { 
					// update the counter
					lineNumber++;
					// if the line is empty, continue searching
		        	if (columns[0].trim().isEmpty()) {
		        		continue;
		        	} else { // if the line is not empty, we have found a line where to start
		        		columnFound = true;
		        		break;
		        	}
				}
			} else { // the first line of the document contains text
				columnFound = true;
			}		
			// if we have found a line with some text
			if (columnFound) {
				// create an attribute map
				XAttributeMap attributeMap = xesFactory.createAttributeMap();
				// get the extension manager
				XExtensionManager extManager = XExtensionManager.instance();
				// add two missing extensions
				extManager.register(XCostExtension.instance());
				extManager.register(XIdentityExtension.instance());
				
				// list of all the attributes defined by the standard extensions
				Collection<XAttribute> attributes = null;   			        			        	
				XExtension extension; 
				int index = 0;
				// for every standard extension of XES
	        	while ((extension = extManager.getByIndex(index)) != null) {
	        		// read the attributes of the extension and add them to the list
	        		if (attributes == null) {
	        			attributes = extension.getEventAttributes();
	        		} else {
	        			attributes.addAll(extension.getEventAttributes());
	        		}
	        		index++;
	        	}
	        	// if we have read some attributes
				if (attributes != null) {
					// for every column element
			    	for (int i=1; i<columns.length; i++) {  
			    		boolean valueFound = false;
			    		// for every attribute of the list of all standard extensions attributes
			    		for(XAttribute attribute : attributes) {  
			    			// look if the column name is equal to an attribute name (it has to be for being valid)
			        		if (columns[i].equals(attribute.getKey())) {
			        			// if we found a match, the column name is valid
			        			valueFound = true;
			        			// we add the attribute to the map of event attributes
			        			attributeMap.put(attribute.getKey(), attribute);
			        			break;
			        		}            			        		
			    		} 
			    		// if we did not find a match between the column name and the attributes, it means that there is an error on the column name
			    		if (!valueFound) {
						    errors = "The file does not contain a valid column name on line " +lineNumber+ ": \n \""
						    						+columns[i]+ "\" \n Please check the file and try again.";
						    return; // logList empty
			    		}
			    	}
				} else { // if we did not read any attributes, is because we can't load and use standard extensions for reading them
				    errors = "Error while checking column names on line " +lineNumber+ ". \n"
				    		+"No registered XES extensions found.";
				    return; // logList empty
				} 
				// here we have an attribute map for the events corresponding to the defined columns in the CSV document
				// create a XLog
				XLog log = xesFactory.createLog();										
				// record the trace id and trace object that we are analysing
				String traceID = null;
		    	XTrace trace = null;    		    	
		    	// check if there is or not another log to analyse in the file
		    	boolean nextLog = false;    		    	
	    		// list to take trace of all generated XID for traces
	    		HashSet<XID> traceXIDList = new HashSet<XID>();
	    		// list to take trace of all generated XID for events
	    		HashSet<XID> eventXIDList = new HashSet<XID>();
	    		
		    	String [] values;
		    	// we start reading all the other lines to convert them in XES elements
		        while ((values = reader.readNext()) != null) { 
		        	// update the counter
		        	lineNumber++;
		        	// if the line is empty, skip to next line
		        	if (values[0].trim().isEmpty()) {
		        		continue;
		        	}
		        	// if the line contains a +, it means we ended reading this log and we have to start reading a new one
		        	if (values[0].trim().equals("+")) {
		        		nextLog = true;
		        		break;
		        	}
		        	// if we are looking at the first trace in the file or to a new one 
					if (traceID == null || !values[0].equals(traceID)) {
						// update the trace id
						traceID = values[0];  
	
						// create a valid XID based on the passed id
						XID traceXID;
						try {
							// transform the id into a XID
							traceXID = XID.parse(traceID);
		    			} catch(RuntimeException e) {
		    				// otherwise generate a new random XID
		    				traceXID = new XID(UUID.randomUUID());
						}
		        		// if this XID has already been generated for a trace in this log
		        		while (traceXIDList.contains(traceXID)) {
		        			// get another XID and try to check again
		        			traceXID = new XID(UUID.randomUUID());
		        		}
		        		// add the unique XID to the xidList
		        		traceXIDList.add(traceXID);
		        		
		        		// clear eventXID list because we have a new trace now
		        		eventXIDList.clear();
					
						// create the id attribute for the trace
						XAttributeID traceXIDAttribute = xesFactory.createAttributeID("identity:id", traceXID, null);
						// create the trace attribute map
						XAttributeMap traceMap = xesFactory.createAttributeMap();
						// add the id attribute to the map
						traceMap.put("identity:id", traceXIDAttribute);
						// create a new trace, using the attribute map just defined
						trace = xesFactory.createTrace(traceMap);
						// add the trace to the log
						log.add(trace);
					}
					// here we have a new trace where to add the events that we read in the file
					// create and event attribute map
					XAttributeMap eventMap = xesFactory.createAttributeMap();			
					// add to the map the attributes that correspond to the column names
					eventMap.putAll(attributeMap); 			
					// fix a limit based on the number of the values in this line
					int limit = values.length;
					// for every column name in the column line
					for (int i=1; i<columns.length; i++) {
						// get the attribute that corresponds to this column
		        		XAttribute attribute = eventMap.get(columns[i]);    
		        		// get the value to add to this attribute
		        		String value;                    		
		        		// if the column index is lower than the value index (for this column there is a value) and if the value is non empty
		        		if ((i < limit) && (!values[i].trim().isEmpty())) {
		        			// read the value corresponding to this column
			        		value = values[i];
		        		} else {
		        			// remove the attribute from the event map because for this event this attribute is not defined
		        			eventMap.remove(columns[i]);
		        			// go to next column
		        			continue;
		        		}
		        		boolean error = false;
		        		// depending on the attribute type, add the value to this attribute (and if an error occurs, show it)
		        		if (attribute instanceof XAttributeLiteral) {
		        			try {
		        				((XAttributeLiteral)attribute).setValue(value);
		        			} catch(RuntimeException e) {
		    					error = true;
		    				}
		    			} else if (attribute instanceof XAttributeDiscrete) {
		    				try {
		    					((XAttributeDiscrete)attribute).setValue(Long.parseLong(value));
		    				} catch(RuntimeException e) {
		    					error = true;
		    				}                    			    				
		    			} else if (attribute instanceof XAttributeContinuous) {
		    				try {
		    					((XAttributeContinuous)attribute).setValue(Double.parseDouble(value));
		    				} catch(RuntimeException e) {
		    					error = true;
		    				}
		    			} else if (attribute instanceof XAttributeTimestamp) {	
		    				// generate a valid Date using the read value and then use it as atribute value
		    				Date timestamp = xsDateTimeConverter.parseXsDateTime(value);
		    				if (timestamp != null) {
		    					((XAttributeTimestamp)attribute).setValue(timestamp);                    			    					
		    				} else {
		    					error = true; 
		    				}                    			    				
		    			} else if (attribute instanceof XAttributeBoolean) {
		    				// look if the value is true or false, otherwise set error
		    				if (value != null && value.equalsIgnoreCase("true")) {
		    					((XAttributeBoolean)attribute).setValue(true);
		    				} else if (value != null && value.equalsIgnoreCase("false")) {
		    					((XAttributeBoolean)attribute).setValue(false);
		    				} else {
		    					error = true;
		    				}                    			    				
		    			} else if (attribute instanceof XAttributeID) {
		    				XID eventXID;
		    				// generate a valid XID starting from the value, or using a new random one
		    				try {
		    					eventXID = XID.parse(value);
		        			} catch(RuntimeException e) {
		        				eventXID = new XID(UUID.randomUUID());
		    				}
			        		// if this XID has already been generated for a trace in this log
			        		while (eventXIDList.contains(eventXID)) {
			        			// get another XID and try to check again
			        			eventXID = new XID(UUID.randomUUID());
			        		}
			        		// add the unique XID to the xidList
			        		eventXIDList.add(eventXID);
		    				// add the value to the attribute
		    				try {
		    					((XAttributeID)attribute).setValue(eventXID);
		    				} catch(RuntimeException e) {
		    					error = true;
		    				}
		    			}		
		        		// if an error occurred while setting the value
		        		if (error) {   
		        			errors = "Error while importing the file. \n"
		        									+ "Malformatted value: line "+lineNumber+", column \""+columns[i]+"\" \n"
		        									+ "Please fix the problem and try again.";
		        			// show the error and stop the process
				    		return;
		        		}                    			        		
		        	}
					// if there are more values than column names, we report this to the user
					if (columns.length < values.length) {
						// set a warning
						String warning = "Line "+lineNumber+" has "+(values.length-columns.length)+" more value/s. \n\n";
						warnings.append(warning);
					}	
					// create an event using the attribute map just defined
		        	XEvent event = xesFactory.createEvent(eventMap);
		        	// add the event to its trace
		        	trace.add(event);
		        }
		        // if we successfully generate an XLog
		        if (!log.isEmpty()) {
		        	// add the XLog to the list
		        	logList.add(log);
		        }
		        // if there is another log to analyse
		        if (nextLog) {
		        	/* call again the method on this file, giving it the list where to
		        	 * add the XLog and the line where to start with this new log */
		        	generateXLogs(reader, logList, lineNumber);
		        }
			}
		} catch (Error e) {
			/* we have to check for possible Errors from the 3rd party library CSVReader
			 * because we cannot correct them in the library code, so we have to handle
			 * them here */
			errors = "Unable to parse the document. The document is no a valid CSV. \n"
				   + "Please correct the document and try again.";
			return;
		}
	}

}
