/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.controller;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.KeyEventDispatcher;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.io.IOException;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.deckfour.xes.in.XParserRegistry;
import org.deckfour.xes.out.XSerializerRegistry;

import unipi.bpm.elm.santeramo.model.EventLogManagerModel;
import unipi.bpm.elm.santeramo.view.EventLogEditor;
import unipi.bpm.elm.santeramo.view.EventLogManagerView;
import unipi.bpm.elm.santeramo.view.FilterPanel;
import unipi.bpm.elm.santeramo.view.FooterAreaButton;
import unipi.bpm.elm.santeramo.view.FootprintFilter;
import unipi.bpm.elm.santeramo.view.FootprintMatrixInfo;
import unipi.bpm.elm.santeramo.view.MenuAreaButton;

/**
 * This class defines the Controller of the Application 
 * (This is the Controller of the MVC pattern).
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 */
public class EventLogManagerController {	
	
	/*
	 * THE VIEW AND THE MODEL
	 */
	
	/**
	 * The <code>EventLogManagerView</code> element 
	 * that represents the View of the application.
	 * 
	 * @see EventLogManagerView
	 */
	private EventLogManagerView elmView;	
	
	/**
	 * The <code>EventLogManagerModel</code> element
	 * that represents the Model of the application.
	 * 
	 * @see EventLogManagerModel
	 */
	private EventLogManagerModel elmModel;
	
	
	/*
	 * LISTENERS
	 */
	
	/**
	 * This object, instance of the inner class
	 * {@link EventLogManagerController.DefaultInternalFrameListener DefaultInternalFrameListener},
	 * will be the <code>JInternalFrameListener</code> for all the <code>JInternalFrame</code> 
	 * frames created.
	 * 
	 * @see JInternalFrame
	 */
	private DefaultInternalFrameListener defaultInternalFrameListener;	
		
	/**
	 * This object, instance of the inner class 
	 * {@link EventLogManagerController.FooterAreaButtonActionListener FooterAreaButtonActionListener},
	 * will be the <code>ActionListener</code> for all the
	 * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton} buttons.
	 * 
	 * @see ActionListener
	 */
	private FooterAreaButtonActionListener footerAreaButtonActionListener;
	
	/**
	 * This object, instance of the inner class 
	 * {@link EventLogManagerController.PopupMenuItemActionListener PopupMenuItemActionListener},
	 * will be the <code>ActionListener</code> for all the <code>JMenuItem</code> items that are
	 * inside the <code>JPopupMenu</code> pop-up menus associated to the 
	 * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton} buttons.
	 * 
	 * @see javax.swing.JMenuItem
	 * @see javax.swing.JPopupMenu
	 * @see ActionListener
	 */
	private PopupMenuItemActionListener popupMenuItemActionListener;
	
	/**
	 * This object, instance of the inner class 
	 * {@link EventLogManagerController.EventLogEditorDocumentListener EventLogEditorDocumentListener},
	 * will be the <code>DocumentListener</code> for all the 
	 * {@link unipi.bpm.elm.santeramo.view.EventLogEditor EventLogEditor} text areas. 
	 * 
	 * @see DocumentListener
	 */
	private EventLogEditorDocumentListener eventLogEditorDocumentListener;
	
	/**
	 * This object, instance of the inner class 
	 * {@link EventLogManagerController.FilterPanelActionListener FilterPanelActionListener},
	 * will be the <code>ActionListener</code> for all the opened
	 * {@link unipi.bpm.elm.santeramo.view.FilterPanel FilterPanel}. 
	 */
	private FilterPanelActionListener filterPanelActionListener;
	
	/**
	 * This object, instance of the inner class 
	 * {@link EventLogManagerController.GenerateEventLogActionListener GenerateEventLogActionListener},
	 * will be the <code>ActionListener</code> for all the dialogs that are created to 
	 * manage the generation of event log from expressions.
	 */
	private GenerateEventLogActionListener generateEventLogActionListener;
	
	/**
	 * This object, instance of the inner class 
	 * {@link EventLogManagerController.SettingsActionListener SettingsActionListener},
	 * will be the <code>ActionListener</code> for all the dialogs that are created to 
	 * show and manage the application settings.
	 */
	private SettingsActionListener settingsActionListener;
	
	
	/*
	 * UTILITY CLASSES
	 */
	
	/**
	 * The <code>EditorManager</code> used by the Controller to manage all the operations
	 * associated with the <code>MenuAreaButton</code> buttons in the application tab menu
	 * that are connected to the management of logs, files and editors. 
	 */
	private EditorManager editorManager;
	
	/**
	 * The <code>AnalysisManager</code> used by the Controller to manage all the operations
	 * associated with the <code>MenuAreaButton</code> buttons in the application tab menu
	 * that are connected to analysis procedures and management. 
	 */
	private AnalysisManager analysisManager;
	
	/**
	 * The <code>InfoManager</code> used by the Controller to manage all the operations
	 * associated with the <code>MenuAreaButton</code> buttons in the application tab menu
	 * that are connected to the application in general (information about the system and
	 * global settings). 
	 */
	private InfoManager infoManager;
	

	
	/*
	 * CONSTRUCTOR
	 */
	
	/**
	 * Constructs a new <code>EventLogMangerController</code>, with the View and
	 * the Model to manage.
	 * 
	 * <p>The constructor then initialises and adds to the View components the 
	 * respective listeners using the {@link #setStandardListeners()} method.
	 * 
	 * <p>At the end, it initialise all the utility classes that it needs.
	 * 
	 * @param elmView the <code>EventLogManagerView</code> object that represents the View 
	 * @param elmModel the <code>EventLogManagerModel</code> object that represents the Model
	 * 
	 * @see EventLogManagerView
	 * @see EventLogManagerModel
	 */
	public EventLogManagerController(EventLogManagerView elmView, EventLogManagerModel elmModel) {		
		// the view and the model
		this.elmView = elmView;
		this.elmModel = elmModel;
		
		// listeners for view components created during the current session
		defaultInternalFrameListener = new DefaultInternalFrameListener();
		footerAreaButtonActionListener = new FooterAreaButtonActionListener();
		popupMenuItemActionListener = new PopupMenuItemActionListener();
		eventLogEditorDocumentListener = new EventLogEditorDocumentListener();
		filterPanelActionListener = new FilterPanelActionListener();
		generateEventLogActionListener = new GenerateEventLogActionListener();
		settingsActionListener = new SettingsActionListener();
		
		// the utility classes
		editorManager = new EditorManager(this);
		analysisManager = new AnalysisManager(this);
		infoManager = new InfoManager(this);
		
		// add custom serializer and parser to the respective registry 
		XSerializerRegistry.instance().register(new XesCsvSerializer());
		XParserRegistry.instance().register(new CsvXesParser());
		
		// add the respective listeners to the standard components of the view
		setStandardListeners();
	}	
	
	
	
	/*
	 * METHODS
	 */	
	
	/**
	 * Starts the application by calling the 
	 * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#start() start()}
	 * method of the View to show the GUI to the user
	 */
	public void start() {
		elmView.start();
	}
	
	/**
	 * Adds to the standard components of the View the respective listeners
	 */
	private void setStandardListeners() {			
		// set the listeners to the main frame
		elmView.addWindowListener(new MainFrameWindowListener());
		elmView.addKeyEventDispatcher(new MainFrameKeyEventDispatcher());
		
		// this listener will be the listener for all the MenuAreaButton buttons, except the Recent one
		MenuAreaButtonActionListener menuAreaButtonActionListener = new MenuAreaButtonActionListener();
		
		Component[] tabComponents = null;
		MenuAreaButton tabButton = null;
		JPanel targetPanel = null;
		// get all the buttons of the fileTab and set the respective listener
		targetPanel = elmView.getFileTab();
		synchronized (targetPanel.getTreeLock()) {
			tabComponents = targetPanel.getComponents(); 	
		}
		for (int i = 0; i < tabComponents.length; i++) {
    		if (tabComponents[i] instanceof MenuAreaButton) {
    			tabButton = (MenuAreaButton)tabComponents[i];
    			if (tabButton.getActionCommand() == "recent") {
    				tabButton.addMouseListener(new RecentButtonMouseListener());
    			} else {
    				tabButton.addActionListener(menuAreaButtonActionListener);
    			}    			
    		}     		
		}	
		// get all the buttons of the discoveryTab and set the respective listener
		targetPanel = elmView.getDiscoveryTab();
		synchronized (targetPanel.getTreeLock()) {
			tabComponents = targetPanel.getComponents(); 
		}
		for (int i = 0; i < tabComponents.length; i++) {
    		if (tabComponents[i] instanceof MenuAreaButton) {
    			tabButton = (MenuAreaButton)tabComponents[i];	
    			tabButton.addActionListener(menuAreaButtonActionListener);
    		}     		
		}		
		// get all the buttons of the helpAndAboutTab and set the respective listener
		targetPanel = elmView.getInfoTab();
		synchronized (targetPanel.getTreeLock()) {
			tabComponents = targetPanel.getComponents(); 
		}	    		
		for (int i = 0; i < tabComponents.length; i++) {
    		if (tabComponents[i] instanceof MenuAreaButton) {
    			tabButton = (MenuAreaButton)tabComponents[i];	
    			tabButton.addActionListener(menuAreaButtonActionListener);
    		}     		
		}
	}
	
	/**
	 * This method returns the <code>EventLogManagerView</code> {@link #elmView} element.
	 * 
	 * @return the <code>elmView</code> component representing the View managed by this Controller.
	 */
	public EventLogManagerView getView() {
		return elmView;
	}
	
	/**
	 * This method returns the <code>EventLogManagerModel</code> {@link #elmModel} element.
	 * 
	 * @return the <code>elmModel</code> component representing the Model managed by this Controller.
	 */
	public EventLogManagerModel getModel() {
		return elmModel;
	}
	
	/**
	 * This method returns the <code>EditorManager</code> {@link #editorManager} element.
	 * 
	 * @return the {@link #editorManager} representing the Editor Manager used by this Controller.
	 */
	public EditorManager getEditorManager() {
		return editorManager;
	}
	
	/**
	 * This method returns the <code>AnalysisManager</code> {@link #analysisManager} element.
	 * 
	 * @return the {@link #analysisManager} representing the Analysis Manager used by this Controller.
	 */
	public AnalysisManager getAnalysisManager() {
		return analysisManager;
	}
	
	/**
	 * This method returns the <code>InfoManager</code> {@link #infoManager} element.
	 * 
	 * @return the {@link #infoManager} representing the Info Manager used by this Controller.
	 */
	public InfoManager getInfoManager() {
		return infoManager;
	}	
	
    /**
     * Asks the View to add a new <code>EventLogEditor</code> internal frame inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea},
     * then adds a new <code>FooterAreaButton</code> button inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#footerArea footerArea},
     * both with the same title.
     * 
     * <p>After that, set the listeners to these two components to implement their connection
     * and functionalities.
     * 
     * <p>At the end, set the content, the associated file and the modified status of the 
     * <code>EventLogEditor</code>, depending on the values of these parameters.
     * 
     * @param editorTitle the title of the editor, <code>null</code> for an auto-generated default title
     * @param editorContent the <code>Reader</code> object opened over the editor content, <code>null</code> for an empty editor
     * @param associatedFilePath the absolute path of the file associated with this editor, 
     * 	<code>null</code> when the editor does not have yet an associated file
     * @param isContentModified if the editor has to be set as modified or not
     * 
     * @see EventLogEditor
     * @see FooterAreaButton
     */
    public void newEventLogEditor(String editorTitle, Reader editorContent, String associatedFilePath, boolean isContentModified) {    	
    	// increase the counter
    	elmModel.addNewInternalFrame();
    	// get the frame identifier
    	int internalFrameID = elmModel.getInternalFrameNumber();
    	// get the title
    	String title = editorTitle;   
    	// if no title selected
    	if (title == null) {  
         	// set the title as a calculated default value
         	title = "Untitled-" + internalFrameID;
    	}
    	// ask the View to create an event log editor with selected title and ID
    	EventLogEditor newEventLogEditor = elmView.newEventLogEditor(title, internalFrameID);
    	// get the editor templates from the Model and add them to the editor
    	newEventLogEditor.addTemplates(elmModel.getEditorTemplates());
    	// get the editor completions from the Model and the templates id and add them to the editor
    	List<String> templateIdList = elmModel.getEditorTemplates().get(0);
    	String[] templatesId = templateIdList.toArray(new String[templateIdList.size()]);
    	newEventLogEditor.addCompletions(elmModel.getEditorCompletions(), templatesId);
     	// if there is a content to set inside the editor
     	if (editorContent != null) {
     		try {
     			// set the content passing the reader to the editor (it will read it and set the text area)
				newEventLogEditor.setTextAreaContent(editorContent);
			} catch (IOException e) {
				// if some error occurred while reading
				elmView.showErrorMessage("Error while loading content in the editor. Please try again.");
			}
     		// start recording edits from now, discarding this insert (its the document base)
     		newEventLogEditor.discardAllEdits();
     	}
     	// if there is an associated file to this editor
     	if (associatedFilePath != null) {
     		newEventLogEditor.setAbsolutePath(associatedFilePath);
     	}
     	// if the editor has been modified 
     	if(isContentModified) {
         	// set the status of the editor as modified
         	newEventLogEditor.setContentModified(true);
    		// modify its titles adding an asterisk at the end of them
            String contentModifiedTitle = newEventLogEditor.getTitle() + "*";	             
            modifyJInternalFrameTitles(newEventLogEditor, contentModifiedTitle);
     	}
     	// set the listeners to the editor
     	newEventLogEditor.addInternalFrameListener(defaultInternalFrameListener);
     	newEventLogEditor.addDocumentListener(eventLogEditorDocumentListener);
     	
     	// ask the View to create a button on the bottom area linked to the just created internal frame, with its same title and ID
     	FooterAreaButton newFooterAreaButton = elmView.newFooterAreaButton(title, internalFrameID);
     	// set the listeners to the button and to its pop-up menu items
     	newFooterAreaButton.addActionListener(footerAreaButtonActionListener);  
     	newFooterAreaButton.addPopupMenuItemsListener(popupMenuItemActionListener);
    } 
    
    /**
     * Asks the View to add a new <code>FootprintMatrixInfo</code> internal frame inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea},
     * then adds a new <code>FooterAreaButton</code> button inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#footerArea footerArea},
     * both with the same title.
     * 
     * <p>After that, set the listeners to these two components to implement their connection
     * and functionalities.
     * 
     * <p>At the end, set the content, the associated file and the exported status of the 
     * <code>FootprintMatrixInfo</code>, depending on the values of these parameters.
     * 
     * @param documentTitle the title of the document, <code>null</code> for an auto-generated default title
     * @param documentContent the content of the document, <code>null</code> for no content
     * @param associatedFilePath the absolute path of the file associated with this footprint matrix, 
     * 	<code>null</code> when the matrix does not have yet an associated file
     * @param isContentExported if the footprint matrix has to be set as exported or not
     *  
     * @see FootprintMatrixInfo
     * @see FooterAreaButton
     */
    public void newFootprintMatrixInfo(String documentTitle, String documentContent, String associatedFilePath, boolean isContentExported) {    	
    	// increase the counter
    	elmModel.addNewInternalFrame();
    	// get the frame identifier
    	int internalFrameID = elmModel.getInternalFrameNumber();
    	// get the title
    	String title = documentTitle;   
    	// if no title selected
    	if (title == null) {  
         	// set the title as a calculated default value
         	title = "Untitled-" + internalFrameID;
    	}
    	// ask the View to create a footprint matrix info page with selected title and ID
    	FootprintMatrixInfo newFootprintMatrixInfo = elmView.newFootprintMatrixInfo(title, internalFrameID);
     	// set the listener to the frame
    	newFootprintMatrixInfo.addInternalFrameListener(defaultInternalFrameListener);
     	// ask the View to create a button on the bottom area linked to the just created internal frame, with its same title and ID
     	FooterAreaButton newFooterAreaButton = elmView.newFooterAreaButton(title, internalFrameID);
     	// set the listeners to the button and to its pop-up menu items
     	newFooterAreaButton.addActionListener(footerAreaButtonActionListener);  
     	newFooterAreaButton.addPopupMenuItemsListener(popupMenuItemActionListener);
     	// if there is a content to set inside the document
     	if (documentContent != null) {
     		newFootprintMatrixInfo.setTextAreaContent(documentContent);    		
     	}
     	// if there is an associated file to this editor
     	if (associatedFilePath != null) {
     		newFootprintMatrixInfo.setAbsolutePath(associatedFilePath);
     	}
     	// if the matrix has been exported 
     	if(isContentExported) {
         	// set the status of the matrix as exported
     		newFootprintMatrixInfo.setContentExported(true);
     	}
    } 
    
    /**
     * Asks the View to add a new <code>JInternalFrame</code> object inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea},
     * then adds a new <code>FooterAreaButton</code> button inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#footerArea footerArea},
     * both with the same title.
     * 
     * <p>After that, set the listeners to these two components to implement their connection
     * and functionalities.
     * 
     * <p>At the end, set the content of the <code>JInternalFrame</code>, if not <code>null</code>.
     * 
     * @param frameTitle the title of the internal frame, <code>null</code> for an auto-generated default title
     * @param frameContent the <code>Component</code> object to add inside the internal frame, <code>null</code> for no content
     * 
     * @see JInternalFrame
     * @see FooterAreaButton
     */
    public void newInternalFrame(String frameTitle, Component frameContent) {
    	// increase the counter
    	elmModel.addNewInternalFrame();
    	// get the frame identifier
    	int internalFrameID = elmModel.getInternalFrameNumber();
    	// get the title
    	String title = frameTitle;   
    	// if no title selected
    	if (title == null) {
         	// set the title as a calculated default value
         	title = "Untitled-" + internalFrameID;
    	}
    	// ask the View to create an internal frame with selected title and ID
    	JInternalFrame newInternalFrame = elmView.newInternalFrame(title, internalFrameID);
     	// set the listener to the internal frame
    	newInternalFrame.addInternalFrameListener(defaultInternalFrameListener);
    	// ask the View to create a button on the bottom area linked to the just created internal frame, with its same title and ID
     	FooterAreaButton newFooterAreaButton = elmView.newFooterAreaButton(title, internalFrameID);
     	// set the listeners to the button and to its pop-up menu items
     	newFooterAreaButton.addActionListener(footerAreaButtonActionListener);  
     	newFooterAreaButton.addPopupMenuItemsListener(popupMenuItemActionListener);
     	// if there is a content to set inside the internal frame
     	if (frameContent != null) {
     		// set the component on the internal frame
     		newInternalFrame.getContentPane().add(frameContent, BorderLayout.CENTER);
     		newInternalFrame.pack();     		
     	}     	    	
    }
    
    /**
     * Sets the title of the target internal frame as the <code>newTitle</code> passed as parameter
     * and then updates all its references in the associated <code>FooterAreaButton</code> to keep
     * the integrity and the association between them.
     * 
     * @param targetInternalFrame the target internal frame to set
     * @param newTitle the new title of the target internal frame and its associated components on the application
     * 
     * @see FooterAreaButton
     * @see JInternalFrame
     */
    public void modifyJInternalFrameTitles(JInternalFrame targetInternalFrame, String newTitle) {
    	if ((targetInternalFrame != null) && (newTitle != null)) {
	    	// search the associated button of the internal frame
			FooterAreaButton footerAreaButton = findFooterAreaButton(targetInternalFrame);
			if(footerAreaButton != null) {
				// set the new text and tool-tip of the associated button
				footerAreaButton.setText(newTitle);
				footerAreaButton.setToolTipText(newTitle);   	    								
			} 
			// now change the title of the internal frame
			targetInternalFrame.setTitle(newTitle);
    	}
    }
    
    /**
     * Searches for the target <code>JInternalFrame</code> inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea} 
     * using its title, returning it or <code>null</code> otherwise.
     * 
     * @param targetID the identifier of the target internal frame to find
     * 
     * @return the target <code>JInternalFrame</code> if the method finds it, <code>null</code> otherwise
     * 
     * @see JInternalFrame
     */
    public JInternalFrame findJInternalFrame(String targetID) {
    	if (targetID != null) { 
	    	// get all opened internal frames
	        JInternalFrame[] internalFrames = elmView.getContentArea().getAllFrames();        
	        // for every internal frame
	        for (JInternalFrame internalFrame: internalFrames) {            	            	
	            if (internalFrame.getName().compareTo(targetID)==0) { // if they are the same              	
	            	// we found it
	            	return internalFrame;
	            }
	        }
	        // if we did not find the internal frame
    	}
        return null;
    }
    
    /**
     * Searches for the associated <code>FooterAreaButton</code> button of the target
     * <code>JInternalFrame</code>, returning it or <code>null</code> otherwise.
     * 
     * @param target the <code>JInternalFrame</code> associated with the button to find
     * 
     * @return the <code>FooterAreaButton</code> button associated to the target <code>JInternalFrame</code>, <code>null</code> otherwise.
     * 
     * @see JInternalFrame
     */
    public FooterAreaButton findFooterAreaButton(JInternalFrame target) {
    	if (target != null) {
	    	// get the internal frame title
	    	String internalFrameID = target.getName();
	    	// get the footerArea
			JPanel footerArea = elmView.getFooterArea();
			// get all the components of the footerArea
			Component[] footerAreaComponents = null;
			synchronized (footerArea.getTreeLock()) {
				footerAreaComponents = footerArea.getComponents(); 
			}	
			// for every component
			for (int i = 0; i < footerAreaComponents.length; i++) {
				// if the component is a FooterAreaButton
	    		if (footerAreaComponents[i] instanceof FooterAreaButton) {
	    			FooterAreaButton footerAreaButton = (FooterAreaButton)footerAreaComponents[i];
	    			/* because there is a connection between the title of an internal frame 
	    			 * and the name of its FooterAreaButton button (they are the same), 
	    			 * we can search the button by using the title of the internal frame. */
	    			if (footerAreaButton.getName().compareTo(internalFrameID) == 0) { // if they are the same 	    				
	    				// we found it
	    				return footerAreaButton;					
					}	
	    		}     		
			}
    	}
		// if we did not find the button
    	return null;
    }
    
    /**
     * Checks if the given URL is reachable online in the given timeout,using an 
     * HTTP connection, and if an HEAD request returns a success/redirect response
     * code or not. 
     * 
     * <p>The method is helpful to check if there is Internet connection in general,
     * to check if a specific resource, specified by its URL, can be accessed through
     * it using an HTTP connection and to check if and HEAD request to this URL returns
     * a response code between 200 and 399. If all of these cases are confirmed, the
     * method will return <code>true</code>.
     * 
     * <p>Either if there is not Internet connection, or if the resource is unreachable,
     * or if the response code given by the resource is not success/redirect, the method 
     * will return <code>false</code> instead.
     * 
     * <p>The possible response code are the following: 
     * <ul>
     * 	<li>under 100 is undetermined</li>
     *  <li>1nn is informal (should not happen on a GET/HEAD)</li>
     *  <li>2nn is success</li>
     *  <li>3nn is redirect</li>
     *  <li>4nn is client error</li>
     *  <li>5nn is server error</li>
     * </ul>
     * 
     * @param url the URL to check
     * @param timeout the timeout for the response
     * 
     * @return <code>true</code> if the method reached the URL in the given time and the response code is between 200 and 399, <code>false</code> otherwise
     */
	public boolean checkInternetConnection(String url, int timeout) {
	    try {
	    	// set the application in a busy state in the meanwhile	    			        
	        elmView.setBusyState(true);  
	        // open an HTTP connection to the URL
	    	HttpURLConnection connection = (HttpURLConnection)new URL(url).openConnection();
	        // set the connection and reading timeout
	    	connection.setConnectTimeout(timeout);
	        connection.setReadTimeout(timeout);
	        // set request method as HEAD request
	        connection.setRequestMethod("HEAD");
	        // get the response code from the connection
	        int responseCode = connection.getResponseCode();
	        // stop the busy state
	        elmView.setBusyState(false); 
	        // if the response code is success or redirect
	        if (200 <= responseCode && responseCode <= 399) {	
	        	// return as resource reached and successful request
	        	return true;
	        } else { // if not
	        	// return as resource not reached
	        	return false;
	        }
	    } catch (IOException exception) { // if a problem occurred during connection or request
	    	// stop the busy state
	    	elmView.setBusyState(false);
	    	// return as resource not reached
	        return false;
	    }	
		 /* Possible response code values:
		  * < 100 is undetermined.
		  * 1nn is informal (shouldn't happen on a GET/HEAD)
		  * 2nn is success
		  * 3nn is redirect
		  * 4nn is client error
		  * 5nn is server error */
	}
	
    /**
     * Asks the View to open a dialog to make the user decide
     * between an empty Event Log or a generation of an Event Log starting
     * from an expression that defines the possible event traces.
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#newEventLog(ActionListener) newEventLog(ActionListener)}
	 * method.
     * 
     * @see EventLogEditor
     * @see EditorManager
     */
    private void newEventLog() {    	
    	editorManager.newEventLog(generateEventLogActionListener);	
    }  
    
    /**
     * Generates a XES log from the passed expression that defines the possible event traces.
     * 
     * The method will first show to the user the generated traces, then ask if he wants to open
     * a new <code>EventLogEditor</code> with the CSV representation of the generated traces, or the
     * complete log in XES standard. 
     * 
     * This method just calls the <code>AnalysisManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.AnalysisManager#generateEventLog(String) generateEventLog(String)}
	 * method. 
     * 
     * @param exp the <code>String</code> representing the expression that defines the possible event traces
     */
    private void generateEventLog(String exp) {
    	analysisManager.generateEventLog(exp);
    }
    
    /**
     * Opens an "Open" dialog to ask the user to select which file to open, then the method tries
     * to open it (.XES and .TXT extensions available).
	 * 
	 * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#openEventLog(String, boolean...) openEventLog(String, boolean...)}
	 * method. 
     *
     * @param selectedFileName the file name to set as selected in the dialog, <code>null</code> for no selection
     * 
     * @see EventLogEditor
     * @see EditorManager
     */
    private void openEventLog(String selectedFileName) {
    	editorManager.openEventLog(selectedFileName);
    }
    
    /**
     * Closes the <code>JInternalFrame</code> passed as parameter, if one is passed,
     * or the selected one inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea},
     * if a frame is set as selected.
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#closeInternalFrame(JInternalFrame) closeInternalFrame(JInternalFrame)}
	 * method. 
     * 
     * @param internalFrame the internal frame to close, <code>null</code> to close the selected internal frame inside the
     *  {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}
     *  
     * @return <code>true</code> if the closing operation successfully close the internal frame, <code>false</code> otherwise 
     * 
     * @see JInternalFrame
     * @see EventLogEditor
     * @see EditorManager
     */
    private boolean closeInternalFrame(JInternalFrame internalFrame) {
    	return editorManager.closeInternalFrame(internalFrame);
    }
    
    /**
     * Saves the content of the <code>EventLogEditor</code> editor (the one selected inside
     * the {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea})
     * in its own file, if an editor is selected. 
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#saveEventLog() saveEventLog()}
	 * method.
     * 
     * @return true if we successfully save the file, false otherwise
     * 
     * @see EventLogEditor
     * @see EditorManager
     */
    private boolean saveEventLog() {
    	return editorManager.saveEventLog();
    }
    
    /**
     * Opens a "Save As" dialog to ask the user to decide where to save the content of
     * the <code>EventLogEditor</code> editor (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}),
     * then it tries to save the content in the selected file. 
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#saveAsEventLog(String) saveAsEventLog(String)}
	 * method.
     * 
     * @param selectedFileName the file name to set as selected in the dialog, <code>null</code> for using the <code>EventLogEditor</code> title
     * 
     * @return true if we successfully save the file, false otherwise 
     * 
     * @see EventLogEditor
     * @see EditorManager
     */
    private boolean saveAsEventLog(String selectedFileName) { 
    	return editorManager.saveAsEventLog(selectedFileName);
     }
    
    /**
     * Opens a "Print" dialog to make the user select the printing settings, then prints
     * the document that is inside the selected <code>EventLogEditor</code> editor (the one
     * inside the {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}). 
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#printDocument() printDocument()}
	 * method.
     * 
     * @return true if we successfully print the document, false otherwise 
     * 
     * @see EventLogEditor
     * @see EditorManager
     */
    private boolean printDocument() {
    	return editorManager.printDocument();
    }
    
    /**
     * Opens an "Import" dialog to ask the user to select which file to import, then the method tries
     * to import it (.CSV and .HTML extensions available).
     *
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#importDocument(String) importDocument(String)}
	 * method.
	 * 
	 * @param selectedFileName the file name to set as selected in the dialog, <code>null</code> for no selection
     * 
     * @return <code>true</code> if we successfully import the file, <code>false</code> otherwise
     * 
     * @see EditorManager
     */
    private boolean importDocument(String selectedFileName) {
    	return editorManager.importDocument(selectedFileName);
    }
    
    /**
     * Opens an "Export" dialog to ask the user to decide where to export (and in which format) 
     * the content either of the selected <code>EventLogEditor</code> editor or of the selected
     * <code>FootPrintMatrixInfo</code> document (that are both inside the
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}). 
     * 
     * <p>The method then tries to export the content of the selected frame in the selected file
     * (CSV format available for the <code>EventLogEditor</code>; HTML, LaTeX, PDF and PNG formats
     * available for the <code>FootPrintMatrixInfo</code>).  
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#exportDocument(String) exportDocument(String)}
	 * method.
     * 
     * @param selectedFileName the file name to set as selected in the dialog, <code>null</code> for no selection
     * 
     * @return <code>true</code> if we successfully export the file, <code>false</code> otherwise
     * 
     * @see EditorManager
     * @see FootprintMatrixInfo
     */
    private boolean exportDocument(String selectedFileName) {
    	return editorManager.exportDocument(selectedFileName);
    }
    
    /**
     * Cuts the selected text in the text area of the <code>EventLogEditor</code> editor
     * (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea})
     * into the system clipboard.
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#cutEventLog() cutEventLog()}
	 * method.
     */
    private void cutEventLog() {
    	editorManager.cutEventLog();
    }
    
    /**
     * Copies the selected text in the text area of the <code>EventLogEditor</code> editor
     * (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea})
     * into the system clipboard.
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#copyEventLog() copyEventLog()}
	 * method.
     */
    private void copyEventLog() {
    	editorManager.copyEventLog();
    }
    
    /**
     * Pastes the content of the system clipboard in the text area of the <code>EventLogEditor</code>
     * editor (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#pasteEventLog() pasteEventLog()}
	 * method.
     */
    private void pasteEventLog() {
    	editorManager.pasteEventLog();
    }
    
    /**
     * Undoes the last action done inside the text area of the the <code>EventLogEditor</code>
     * editor (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#undoEventLog() undoEventLog()}
	 * method.
     */
    private void undoEventLog() {
    	editorManager.undoEventLog();
    }
    
    /**
     * Redoes the last action undone inside the text area of the the <code>EventLogEditor</code>
     * editor (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#redoEventLog() redoEventLog()}
	 * method.
     */
    private void redoEventLog() {
    	editorManager.redoEventLog();
    }
    
    /**
     * Increases by 1pt the font size of the content of the <code>EventLogEditor</code> editor
     * (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#zoomInEventLog() zoomInEventLog()}
	 * method.
     * 
     * @see EventLogEditor
     */
    private void zoomInEventLog() {
    	editorManager.zoomInEventLog();
    }
    
    /**
     * Decreases by 1pt the font size of the content of the <code>EventLogEditor</code>
     * editor (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#zoomOutEventLog() zoomOutEventLog()}
	 * method.
     * 
     * @see EventLogEditor
     */
    private void zoomOutEventLog() {
    	editorManager.zoomOutEventLog();
    }
    
    /**
     * Reset to the default value the font size of the content of the <code>EventLogEditor</code>
     * editor (the one selected inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>This method just calls the <code>EditorManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.EditorManager#resetZoomEventLog() resetZoomEventLog()}
	 * method.
     * 
     * @see EventLogEditor
     */
    private void resetZoomEventLog() {
    	editorManager.resetZoomEventLog();
    }
    
    /**
     * Validates the content of the <code>EventLogEditor</code> editor (the one selected
     * inside the {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}),
     * checking if its content respects the XES XSD Schema and therefore it is a valid XES document.
     * 
     * <p>This method just calls the <code>AnalysisManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.AnalysisManager#validateEventLog() validateEventLog()}
	 * method.
     * 
     * @return true if we successfully validate the file, false otherwise
     * 
     * @see EventLogEditor
     * @see AnalysisManager
     */
    private boolean validateEventLog() {
    	return analysisManager.validateEventLog();
    }
    
    /**
     * This method analyses the XES log contained inside the selected <code>EventLogEditor</code> (or the passed one as parameter),
     * using the <code>EventLogEditor</code> filter (or the one passed as parameter), creating at the end of the process a HTML report
     * with a footprint matrix in it representing the casual dependencies between event traces. 
     * 
     * <p>The report will be eventually showed to the user using a <code>FootprintMatrixInfo</code>, if asked by using the relative parameter.
     * 
     * <p>If the editor filter is not complete while analysing the document, the process, instead of using that filter (because not complete),
     * records all the needed information and updates the filter to make it complete for the next analysis.
     * 
     * <p>This method just calls the <code>AnalysisManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.AnalysisManager#generateEventLogFootprint(boolean, EventLogEditor, FootprintFilter...) generateEventLogFootprint(boolean, EventLogEditor, FootprintFilter...)}
	 * method.
     * 
     * @param showMatrix <code>true</code> to open a <code>FootprintMatrixInfo</code> with the result of the analysis, <code>false</code> to make the analysis and save the result, but without showing them to the user
     * @param editor the <code>EventLogEditor</code> containing the XES log to analyse, <code>null</code> to use the selected editor inside the application
     * @param filter the <code>FootprintFilter</code> to use for the analysis, nothing (this is a optional parameter) to use the editor filter.
     * 
     * @see EventLogEditor
     * @see AnalysisManager
     * @see FootprintFilter
     * @see unipi.bpm.elm.santeramo.view.FootprintMatrixInfo
     */
    private void generateEventLogFootprint(boolean showMatrix, EventLogEditor editor, FootprintFilter... filter) {    	
    	if (filter.length > 0) {
    		analysisManager.generateEventLogFootprint(showMatrix, editor, filter);
    	} else {
    		analysisManager.generateEventLogFootprint(showMatrix, editor);
    	}
    	
	}
 
    /**
     * Shows a filters dialog associated with the <code>EventLogEditor</code> editor 
     * (the selected one inside the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * <p>The dialog content will be calculated based on the editor
     * <code>FootprintFilter</code>, to show to the user the possible selections.
     * 
     * <p>A {@link FilterPanelActionListener} will be associated to the dialog buttons
     * to implement the panel functions.
     * 
     * This method just calls the <code>AnalysisManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.AnalysisManager#setFootprintFilters(ActionListener) setFootprintFilters(ActionListener)}
	 * method.
	 */
    private void setFootprintFilters() {    	
    	analysisManager.setFootprintFilters(filterPanelActionListener);
	}   
    
    /**
     * Generates a XES log from the content (in a CSV-format) of the selected <code>EventLogEditor</code> 
     * (the one inside the {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}).
     * 
     * The method will open a new <code>EventLogEditor</code> editor with the generated log inside of it.
     * 
     * This method just calls the <code>AnalysisManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.AnalysisManager#generateXesFromCsv() generateXesFromCsv()}
	 * method.
	 */
    private void generateXesFromCsv() {
    	analysisManager.generateXesFromCsv();
    }
	
    /**
     * Shows the application settings to manage all the available values
     * for the application, the <code>EventLogEditor</code> and the XES Standard. 
     * This method just calls the <code>InfoManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.InfoManager#showSettings(ActionListener) showSettings(ActionListener)} method.
	 */
    private void showSettings() {
    	infoManager.showSettings(settingsActionListener);
    }
    
    /**
     * Shows the application credits (About).
     * 
     * This method just calls the <code>InfoManager</code>
	 * {@link unipi.bpm.elm.santeramo.controller.InfoManager#showAbout() showAbout()} method.
	 */
    private void showAbout() {
    	infoManager.showAbout();
    }
	
	
    /*
     * INNER CLASSES
     */ 
    
    /**
     * This inner class extends <code>WindowAdapter</code> for creating
     * a specific listener for the main <code>JFrame</code>. 
     *  
     * <p>It defines the behaviour of the <code>JFrame</code> when it is in the process of
     * being closed. The window first asks if the user is sure to close the application,
     * then it checks if there are some unsaved <code>JInternalFrame</code> frames in the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}
     * and asks the user if he wants to save them or not before closing them. At the end of 
     * this process the window will close itself only if all the internal frames got closed 
     * successfully.
     *  
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see WindowAdapter
     * @see JFrame  
     * @see JInternalFrame
     */
    class MainFrameWindowListener extends WindowAdapter {
    	
    	/**
    	 * Invoked when a window is in the process of being closed. The close operation
    	 * can be overridden at this point.
    	 * 
    	 * <p>The method first asks if the user is sure to close the application,
	     * then it checks if there are some unsaved <code>JInternalFrame</code> frames in the 
	     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea}
	     * and asks the user if he wants to save them or not before closing them. At the end of 
	     * this process the method will set the close operation of the window as <code>EXIT_ON_CLOSE</code>
	     * only if all the internal frames got closed successfully, as <code>DO_NOTHING_ON_CLOSE</code> 
	     * when the first error occurs otherwise, stopping the closing process of the internal frames.
    	 * 
    	 * @param e the <code>WindowEvent</code> generated by the window
    	 * 
    	 * @see WindowEvent
    	 */
    	@Override
    	public void windowClosing(WindowEvent e) {
    		// retrieve the user's choice		    			
			String message = "Do you really want to close the application?";
			String title = "Close";
			Object[] options = {"Yes", "   No   "};
			int choice = elmView.showOptionDialog(message, title, JOptionPane.QUESTION_MESSAGE, options, options[0]);
			// check user's decision
			if (choice == 0) { // if yes
				boolean savingFlag = true;
				// get all opened internal frames
		        JInternalFrame[] openedInternalFrames = elmView.getContentArea().getAllFrames();        
		        // for every internal frame
		        for (JInternalFrame internalFrame: openedInternalFrames) {  
		        	try {
		        		// set the internal frame that we are going to close as selected
						internalFrame.setSelected(true);
					} catch (PropertyVetoException ex) {} // we do not need to do anything if an exception has been thrown, 
														  // just catch it here to not cause an application error. 
														  //We will do something on the next lines to manage a possible failure of the try code
		        	// call the close operation and get the result
		        	savingFlag = closeInternalFrame(internalFrame);		        	
		        	// if we did not close the internal frame successfully
		        	if (!savingFlag) {
		        		// stop the closing process
		        		break;
		        	}
		        	// otherwise go to the next internal frame
		        }	
		        // at the end of the cycle, we check if we successfully close all the internal frames or not
				if (savingFlag) {
					// set the close operation of the window as exit
					elmView.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	
				} else {
					// set the close operation of the window as do not close
					elmView.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
				}					
			} else { //if no or the user close the dialog
				elmView.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			}
    	}
    }
    
    /**
     * This inner class implements <code>KeyEventDispatcher</code> to realise 
     * a specific key event dispatcher to give to the <code>KeyboardFocusManager</code>
     * of the main <code>JFrame</code>.
     * 
     * <p>This dispatcher gets all the <code>KEY_PRESSED</code> events and checks if they are
     * some specific shortcuts used to call the buttons of the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView EventLogManagerView}.
     * 
     * <p>If they are, this class calls the correct function connected to those specific
     * shortcuts.
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see KeyEventDispatcher
     * @see java.awt.KeyboardFocusManager
     * @see JFrame
     */
    class MainFrameKeyEventDispatcher implements KeyEventDispatcher {
        
    	/**
    	 * This method is called by the current KeyboardFocusManager requesting 
    	 * that this KeyEventDispatcher dispatch the specified event on its behalf.
    	 * 
	     * <p>This methods gets all the <code>KEY_PRESSED</code> events and checks if they are
	     * some specific shortcuts used to call the buttons of the 
	     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView EventLogManagerView}.
	     * 
	     * <p>If they are, this method calls the correct function connected to those specific
	     * shortcuts.
    	 * 
    	 * @param e the <code>KeyEvent</code> to dispatch
    	 * 
    	 * @return <code>true</code> if the <code>KeyboardFocusManager</code> should take no further action
    	 * 			with regard to the <code>KeyEvent</code>; false otherwise
    	 * 
    	 * @see java.awt.KeyboardFocusManager
    	 * @see KeyEvent
    	 */
    	@Override
        public boolean dispatchKeyEvent(KeyEvent e) {
            // if the key event is a KEY_PRESSED
    		if (e.getID() == KeyEvent.KEY_PRESSED) {           	
            	// check if the key event is one of these shortcuts, then apply the connected function    			
            	if ((e.getKeyCode() == KeyEvent.VK_N) && e.isControlDown()) { // CTRL+N            		
            		newEventLog();             	
                } else if ((e.getKeyCode() == KeyEvent.VK_O) && e.isControlDown()) { // CTRL+O
                	openEventLog(null); // null = no file selected in the dialog              
                } else if ((e.getKeyCode() == KeyEvent.VK_W) && e.isControlDown()) { // CTRL+W
                	closeInternalFrame(null); // null = close the selected internal frame inside the contentArea
                } else if ((e.getKeyCode() == KeyEvent.VK_S) && e.isControlDown() && !e.isAltDown()) { // CTRL+S
                	saveEventLog();
                } else if ((e.getKeyCode() == KeyEvent.VK_S) && e.isControlDown() && e.isAltDown()) { // CTRL+ALT+S
                	saveAsEventLog(null); // null = internal frame title as selected file in the dialog
                } else if ((e.getKeyCode() == KeyEvent.VK_P) && e.isControlDown()) { // CTRL+P
                	printDocument();
                } else if ((e.getKeyCode() == KeyEvent.VK_I) && e.isControlDown()) { // CTRL+I
                	importDocument(null);  
                } else if ((e.getKeyCode() == KeyEvent.VK_E) && e.isControlDown()) { // CTRL+E
                	exportDocument(null);                	
                } else if ((e.getKeyCode() == KeyEvent.VK_Z) && e.isControlDown()) { // CTRL+Z
                	undoEventLog();             	
                } else if ((e.getKeyCode() == KeyEvent.VK_Y) && e.isControlDown()) { // CTRL+Y
                	redoEventLog();             	
                } else if ((e.getKeyCode() == KeyEvent.VK_ADD) && e.isControlDown()) { // CTRL+Num+
                	zoomInEventLog();         	
                } else if ((e.getKeyCode() == KeyEvent.VK_SUBTRACT) && e.isControlDown()) { // CTRL+Num-
                	zoomOutEventLog();            	
                } else if ((e.getKeyCode() == KeyEvent.VK_0) && e.isControlDown()) { // CTRL+0
                	resetZoomEventLog();               	
                } else if ((e.getKeyCode() == KeyEvent.VK_Q) && e.isControlDown()) { // CTRL+Q
                	validateEventLog();	
                } else if ((e.getKeyCode() == KeyEvent.VK_R) && e.isControlDown()) { // CTRL+R
                	generateEventLogFootprint(true, null); // print matrix and use the selected editor
                } else if ((e.getKeyCode() == KeyEvent.VK_T) && e.isControlDown()) { // CTRL+T
                	setFootprintFilters();
                } else if ((e.getKeyCode() == KeyEvent.VK_G) && e.isControlDown()) { // CTRL+G
                	generateXesFromCsv();
    			} else if ((e.getKeyCode() == KeyEvent.VK_H) && e.isControlDown()) { // CTRL+H
    				showSettings();
                } else if ((e.getKeyCode() == KeyEvent.VK_F1)) { // F1
                	showAbout();
                }
            }
            // the KeyEvent is passed to the next KeyEventDispatcher in the chain
            return false;
        }
    }
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the <code>JButton</code> buttons of the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#menuArea menuArea}. 
     * 
     * <p>It defines the behaviour of all the 
     * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#menuArea menuArea}
     * buttons, implementing the respecting action.
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see ActionListener
     * @see ActionEvent  
     * @see JButton
     */
    class MenuAreaButtonActionListener implements ActionListener {

    	/**
    	 * Invoked when an action occurs. This method checks, using the respective 
    	 * action command, which component threw the event and then invokes the
    	 * right method of the Controller for implementing the respecting action.
    	 * 
    	 * @param e the action event generated by a component
    	 * 
    	 * @see ActionEvent
    	 */
    	@Override
        public void actionPerformed(ActionEvent e) {    		
    		// get the action command of the JButton button that generates the ActionEvent e
            String actionCommand = e.getActionCommand();
            
            // check the action command and do the corresponding action
            switch (actionCommand) {
	            case "new":  		newEventLog();
	                     			break;
	            case "open":  		openEventLog(null); // null = no file selected in the dialog
	                     			break;
	            case "close":		closeInternalFrame(null); // null = close the selected internal frame inside the contentArea
									break;
	            case "save":  		saveEventLog();
									break;
	            case "save-as":  	saveAsEventLog(null); // null = internal frame title as selected file in the dialog
	                     			break;
	            case "print":  		printDocument();
									break;
	            case "import": 		importDocument(null); // null = no file selected in the dialog
									break;
	            case "export": 		exportDocument(null); // null = no file selected in the dialog
									break;
	            case "cut": 		cutEventLog();
									break;
	            case "copy": 		copyEventLog();
									break;
	            case "paste": 		pasteEventLog();
									break;
	            case "undo": 		undoEventLog();
									break;
	            case "redo": 		redoEventLog();
									break;
	            case "zoom-in": 	zoomInEventLog();
									break;
	            case "zoom-out": 	zoomOutEventLog(); 
									break;
	            case "reset-zoom": 	resetZoomEventLog(); 
									break;
	            case "validate":  	validateEventLog();
     								break;
	            case "footprint":  	generateEventLogFootprint(true, null); // print matrix and use the selected editor
	                     			break;
	            case "filters": 	setFootprintFilters();
	                     			break;
	            case "generate":	generateXesFromCsv();
	            					break;
	            case "settings": 	showSettings();
     								break;
	            case "about":  		showAbout();
									break;
	            default: 			break;
            }         
        }
    }
    
    /**
     * This inner class extends <code>MouseAdapter</code> for creating a 
     * specific listener for the <code>MenuAreaButton</code> "Recent" button.  
     * 
     * <p>When the button get pressed, this listener will ask the View to show a pop-up menu connected
     * with this button and containing a set of menu items that represent the elements
     * in the {@link unipi.bpm.elm.santeramo.model.EventLogManagerModel#recentLogList recentLogList}
     * of the Model. These elements are the most recent document opened by the application.
     * 
     * <p>To each of these items, the View will add a {@link RecentButtonItemActionListener}, to manage their
     * behaviour when pressed.
     * 
     * @see MouseAdapter
     * @see MenuAreaButton
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     */
    class RecentButtonMouseListener extends MouseAdapter {
    	
    	/**
    	 * This special <code>ActionListener</code> will be associated to every items
    	 * of the pop-up menu created by the {@link RecentButtonMouseListener#mousePressed(MouseEvent)}
    	 * and will manage their corresponding action.
    	 */
    	private RecentButtonItemActionListener itemListener;
    	
    	
    	/**
    	 * Creates a <code>RecentButtonMouseListener</code>, calling the
    	 * constructor of the super class and then initialising the instance 
    	 * variables.
    	 */
    	public RecentButtonMouseListener() {
    		super();
    		// set variables
    		itemListener = new RecentButtonItemActionListener();
    	}
    	
    	/**
    	 * Invoked when a mouse button has been pressed on a component
    	 * 
    	 * <p>This method asks the view to shop a pop-up menu connected with the selected <code>JButton</code>,
    	 * containing a list defined by the passed <code>LinkedList</code> read from the Model and with
    	 * the passed <code>ActionListener</code> associated with every item in the list.
    	 */
    	@Override
		public void mousePressed(MouseEvent e) {
    		if (elmModel.hasRecentLogs()) {
        		// ask the View to show a pop-up menu associated with the selected button and with the passed list and listener
        		elmView.showPopup((JButton)e.getSource(), elmModel.getRecentLogs(), itemListener); 
    		}
		}
    }    
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the 
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the <code>JMenuItem</code>
     * items of the <code>JPopupMenu</code> pop-up added to an 
     * {@link unipi.bpm.elm.santeramo.view.MenuAreaButton MenuAreaButton} button by the
     * {@link RecentButtonMouseListener} listener.
     * 
     * <p>The implemented method gets from the selected <code>JMenuItem</code> its action command
     * (that represents a path to a file) and ask the {@link EventLogManagerController#editorManager}
     * to open the corresponding file.
     * 
     * @see ActionListener
     * @see javax.swing.JMenuItem
     * @see javax.swing.JPopupMenu
     * @see MenuAreaButton
     * @see RecentButtonMouseListener
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     */
    class RecentButtonItemActionListener implements ActionListener {
    	
    	/**
    	 * Invoked when an action occurs.
    	 * 
    	 * <p>The method gets from the <code>JMenuItem</code> that generates the event its action command
    	 * (that represents a path to a file) and ask the {@link EventLogManagerController#editorManager}
    	 * to open the corresponding file.
    	 */
    	@Override
		public void actionPerformed(ActionEvent e) {	
    		// ask the editor manager to open the file associated with the passed path 
			editorManager.openEventLog(e.getActionCommand(), true);		  			
		}
    }
    
    /**
     * This inner class extends <code>InternalFrameAdapter</code> for creating a 
     * specific listener for the <code>JInternalFrame</code> frames. 
     *  
     * <p>It defines the behaviour of a <code>JInternalFrame</code> frame when it
     * changes its status to maximised, minimised, activated or closed, overriding the 
     * respective methods of the <code>InternalFrameAdapter</code> class.
     *  
     * <p>When the <code>JInternalFrame</code> frame gets minimised, it disappears
     * completely from the <code>JDesktopPane</code> panel; when it get maximised,
     * it appears again. When it gets closed, we delete also its 
     * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton} button,
     * when it gets activated, we set as selected also its 
     * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton} button.
     *  
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see InternalFrameAdapter
     * @see InternalFrameEvent  
     * @see JInternalFrame
     * @see JDesktopPane
     */
    class DefaultInternalFrameListener extends InternalFrameAdapter {
    	  	
    	/**
    	 * Invoked when an internal frame is iconified.
    	 * 
    	 * <p>This method set as not visible the <code>JInternalFrame</code> frame
    	 * that generates this event, making it invisible inside the 
    	 * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea} 
    	 * 
    	 * @param e the event generated by the <code>JInternalFrame</code> frame
    	 * 
    	 * @see JInternalFrame
    	 * @see InternalFrameEvent
    	 */
    	@Override
    	public void internalFrameIconified(InternalFrameEvent e) {    		
    		/* setting the internal frame not visible, we will make the 
    		 * minimised internal frame disappearing from the contentArea */ 
    		e.getInternalFrame().setVisible(false);
        }
    	
    	/**
    	 * Invoked when an internal frame is de-iconified.
    	 * 
    	 * <p>This method set as visible the <code>JInternalFrame</code> frame
    	 * that generates this event, making it visible inside the 
    	 * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea},
    	 * and it also tries to set the frame as selected. 
    	 * 
    	 * @param e the event generated by the <code>JInternalFrame</code> frame
    	 * 
    	 * @see JInternalFrame
    	 * @see InternalFrameEvent
    	 */
    	@Override
    	public void internalFrameDeiconified(InternalFrameEvent e) {    		
    		// set the internal frame visible
    		e.getInternalFrame().setVisible(true);
        	
        	try {
        		// set the internal frame selected
        		e.getInternalFrame().setSelected(true);                   		
    		} catch (PropertyVetoException exception) {
            	elmView.showErrorMessage("The application was not able to select the window. Please try again.");
            } 
    	}
    	
    	/**
    	 * Invoked when an internal frame is in the process of being closed. The close operation 
    	 * can be overridden at this point.
    	 * 
         * <p>This method first checks if the internal frame is an <code>EventLogEditor</code>, in that
         * case it looks if this frame has been modified by the user or not: in the first case, it 
         * asks if the user wants to save or not the document before closing; in the second one, checks if the
         * associated file still exists and decides to close or to ask to save again the document depending on this.
         * 
         * <p>If the internal frame is not an <code>EventLogEditor</code>, the method then checks if is a <code>FootprintMatrixInfo</code>.
         * If it is, the method checks if the document has already been exported or not: in the first case, it checks if the associated file
         * still exists and decides to close or to ask to export again the document depending on this; in the second one, it asks directly 
         * if the user wants to export or not the document before closing.
         * 
         * <p>If the frame to close is not an <code>FootprintMatrixInfo</code> either, the method closes directly the internal frame, because
         * there is not any specific action to do before closing for a generic internal frame.
         * 
         * <p>This method decides the closing or not of the target internal frame, depending on these cases, and it will override the close
         * operation of the frame accordingly (setting it as <code>DISPOSE_ON_CLOSE</code> or <code>DO_NOTHING_ON_CLOSE</code>).
    	 * 
    	 * @param e the event generated by the <code>JInternalFrame</code> frame
    	 * 
    	 * @see EventLogEditor
    	 * @see JInternalFrame
    	 * @see InternalFrameEvent
    	 */
    	@Override
    	public void internalFrameClosing(InternalFrameEvent e) { 
    		// take the selected internal frame to close
    		JInternalFrame selectedInternalFrame = e.getInternalFrame();
    		// if the frame is an EventLogEditor
    		if(selectedInternalFrame instanceof EventLogEditor) {
    			EventLogEditor eventLogEditor = (EventLogEditor)selectedInternalFrame;
    			// default settings
    			boolean fileClosed = false;
				String location = null;
				String message = "Do you want to save the document before closing?";
				// get the absolute path of the file associated with this editor
				String absolutePath = eventLogEditor.getAbsolutePath();   	
				// if the editor content has not been modified by the user, check the associated file and then decide
    			if (!eventLogEditor.isContentModified()) {
    				if ((absolutePath == null)||(absolutePath.equals(""))) {
						// this is a new empty event log, so we can just close it
    					eventLogEditor.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
    					fileClosed = true;
					} else {  	
						// create a Path object using the String path
						Path targetFile = Paths.get(absolutePath);	
						// if the file still exists 
						if (Files.exists(targetFile)) {
							// the editor has no modifications to save and its associated file still exists, so we can close it
							eventLogEditor.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);	
							fileClosed = true;
						} else if (Files.notExists(targetFile)) { // if the target file does not longer exist	
							// set a specific message for the save procedure and the name for the file
							message = "The file associated with this editor does not longer exist in your file system. \n" 
    								+ "Do you want to save the document in a new file before closing?";
							location = targetFile.getFileName().toString();
						} else { //if the process cannot determine if the file exists or not, because it does not have access to the file
							// show an error message to the user and then set the saving name for the file
							elmView.showErrorMessage("The application cannot access the file associated with this editor in your file system and check it. \n"
									+ "We advise to save the document in a new file before closing it, to not loose the editor content.");
							location = targetFile.getFileName().toString();
						}
					}
    			}    	
				// if the file is still open, it means we need to save it
				if(!fileClosed) {
					// retrieve the user's choice, using the message decided before	    			
	    			String title = "Save";
	    			// if we have to save in a new location
	    			if (location != null) {
	    				title = title + " As";
	    			}
	    			Object[] options = {"Yes", "No", "   Cancel   "};
    				int choice = elmView.showOptionDialog(message, title, JOptionPane.QUESTION_MESSAGE, options, options[0]);
    				// check user's decision
    				if (choice == 0) { // if YES, call the save method
    					boolean fileSaved;
    					// if we have to save in a new location
    					if (location != null) {
    						fileSaved = saveAsEventLog(location);
    					} else { // otherwise just save in the same location
    						fileSaved = saveEventLog();
    					}	    					
    					if (fileSaved) { // if we have successfully saved the file, then set the editor as closable
    						eventLogEditor.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
    					} else { // if we have not saved the file, then do not close the editor because its content has not been saved
    						eventLogEditor.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE); 
    					}    					
    				} else if (choice == 1) { // if NO, just set the internal frame as closable, without saving it
    					eventLogEditor.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
    				} else { // if CANCEL (2) or the user close the dialog (-1), set the frame as not closable
    					eventLogEditor.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);  
    				}			
				}
    		} else if (selectedInternalFrame instanceof FootprintMatrixInfo) { // if the frame is a FootprintMatrixInfo
				FootprintMatrixInfo footprintMatrixInfo = (FootprintMatrixInfo)selectedInternalFrame;    			   			
    			// default export message
				boolean fileClosed = false; 
				String message = "Do you want to export the document before closing? \n"
								+ "(Export the document in HTML format if you want to open it again in this application)";
				String location = null;
				// check if the file has already been exported				
				if (footprintMatrixInfo.isContentExported()) { // if the document has already been exported by the user, check the associated file and then decide
					// create a Path object using the String path
					Path targetFile = Paths.get(footprintMatrixInfo.getAbsolutePath());	   		
					// if the file still exists 
					if (Files.exists(targetFile)) {
						// it has been exported and its associated file still exists, so we can close it
						footprintMatrixInfo.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
						fileClosed = true;	
					} else if (Files.notExists(targetFile)) { // if the target file does not longer exist	
						// set a specific message for the export procedure and the name for the file    			
			    		message = "The HTML file associated with this document does not longer exist in your file system. \n" 
			    							+ "Do you want to export the document in a new HTML file before closing? \n"
			    							+ "(Export the document in HTML format if you want to open it again in this application)";
			    		location = targetFile.getFileName().toString();
					} else { //if the process cannot determine if the file exists or not, because it does not have access to the file
						// show an error message to the user, then set a specific message for the export procedure and the export name for the file
						elmView.showErrorMessage("The application cannot access the HTML file associated with this editor in your file system and check it. \n"
									+ "We advise to export the document in a new HTML file before closing it, to not loose this content.");
						message = "Do you want to export the document before closing?";
						location = targetFile.getFileName().toString();
					}
				}
				// if the file is still open, it means we need to export it
				if (!fileClosed) {	
					// retrieve the user's choice, using the message decided before	        			
	    			String title = "Export As";
	    			Object[] options = {"Yes", "No", "   Cancel   "};
					int choice = elmView.showOptionDialog(message, title, JOptionPane.QUESTION_MESSAGE, options, options[0]);
					// check user's decision
					if (choice == 0) { // if YES, call the export method
						// if we have to export in a new location or in the same is decided by the value of location (null or String)
						boolean fileExported = exportDocument(location);
						if (fileExported) { // if we have successfully exported the file, then set the editor as closable
							selectedInternalFrame.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
						} else { // if we have not exported the file, then do not close the editor because its content has not been saved
							selectedInternalFrame.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE); 
						}    					
					} else if (choice == 1) { // if NO, just set the internal frame as closable, without exporting it
						selectedInternalFrame.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
					} else { // if CANCEL (2) or the user close the dialog (-1), set the frame as not closable
						selectedInternalFrame.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);  
					} 
				}
    		} else { // for every other type of frame, close them directly		
    			selectedInternalFrame.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
    		}
    	}
    	
    	/**
    	 * Invoked when an internal frame has been closed.
    	 * 
    	 * <p>This method removes the <code>JInternalFrame</code> frame that generates this event from the
    	 * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea},
    	 * then it searches the <code>FooterAreaButton</code> button linked to
    	 * this internal frame and removes it from the 
    	 * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#footerArea footerArea}.
    	 * 
    	 * @param e the event generated by the <code>JInternalFrame</code> frame
    	 * 
    	 * @see JInternalFrame
    	 * @see InternalFrameEvent
    	 * @see FooterAreaButton
    	 */
    	@Override
    	public void internalFrameClosed(InternalFrameEvent e) {     		
    		// take the selected internal frame
    		JInternalFrame selectedInternalFrame = e.getInternalFrame();
 			// remove the internal frame from the content pane
    		JDesktopPane contentArea = elmView.getContentArea();
    		contentArea.remove(selectedInternalFrame);
    		contentArea.revalidate();
    		contentArea.repaint();   
    		// if the closed internal frame was an event log editor
    		if(selectedInternalFrame instanceof EventLogEditor) {
    			// get the editor
    			EventLogEditor eventLogEditor = (EventLogEditor)selectedInternalFrame;
    			// if it had an associated filter dialog, close it (the method does all the checks)
    			eventLogEditor.closeFilterDialog();
    		}    		
    		// search the associated button of the internal frame
    		FooterAreaButton footerAreaButton = findFooterAreaButton(selectedInternalFrame);
    		if(footerAreaButton != null) {
    			// get the footerArea and the buttons group    						
    			JPanel footerArea = elmView.getFooterArea();
    			ButtonGroup footerAreaButtonsGroup = elmView.getFooterAreaButtonsGroup();    
    			// remove the button from the footerArea
				footerArea.remove(footerAreaButton);
				// remove the button from the group
				footerAreaButtonsGroup.remove(footerAreaButton);
				// re-validate and repaint the footerArea without the button
				footerArea.revalidate();
				footerArea.repaint();						
    		}    		
    	}
    	
    	/**
    	 * Invoked when an internal frame is activated.
    	 * 
    	 * <p>This method searches the <code>FooterAreaButton</code> button linked to
    	 * the <code>JInternalFrame</code> frame that generates this event and sets it as selected in the 
    	 * {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#footerArea footerArea}.
    	 * 
    	 * @param e the event generated by the <code>JInternalFrame</code> frame
    	 * 
    	 * @see JInternalFrame
    	 * @see InternalFrameEvent
    	 * @see FooterAreaButton 
    	 */
    	@Override
    	public void internalFrameActivated(InternalFrameEvent e) {    		
    		// search the associated button of the internal frame
    		FooterAreaButton footerAreaButton = findFooterAreaButton(e.getInternalFrame());
    		if(footerAreaButton != null) {
    			footerAreaButton.setSelected(true);
    		}    		
    	}
	}
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the 
     * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton} buttons.
     *  
     * <p>Every {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton}
     * button is connected to a specific <code>JInternalFrame</code> frame through this
     * listener that gives to the button the possibility to minimise, maximise or select 
     * the connected frame by simply pressing it.
     *  
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see ActionListener
     * @see ActionEvent  
     * @see FooterAreaButton
     * @see JInternalFrame
     */
    class FooterAreaButtonActionListener implements ActionListener {

    	/**
    	 * Invoked when an action occurs. This method checks which <code>FooterAreaButton</code>
    	 * button generates the action event, then it checks the state of the linked <code>JInternalFrame</code>
    	 * frame and implements the respective action to apply to this frame, according to its state.
    	 * 
    	 * <p>If the frame is minimised, the method maximises it; if the frame is maximised and selected,
    	 * the method minimises it; if the frame is maximised but not selected, the method set it as selected.
    	 * 
    	 * @param e the action event generated by a component
    	 * 
    	 * @see JInternalFrame
    	 * @see ActionEvent
    	 * @see FooterAreaButton 
    	 */
    	@Override
        public void actionPerformed(ActionEvent e) {	    		    		
    		// get the text of the button that generates the ActionEvent e
            String internalFrameID = ((AbstractButton)e.getSource()).getName();            
			/* because there is a connection between the name of an internal frame 
			 * and the name of its button (they are the same), we can search the 
			 * right internal frame to manage by using the name of its button */
            JInternalFrame internalFrame = findJInternalFrame(internalFrameID);
       
            if (internalFrame != null) {
	            if (internalFrame.isIcon()) { // if the internal frame is minimised
	            	try {
	            		// set the internal frame maximised
	            		internalFrame.setIcon(false);
					} catch (PropertyVetoException exception) {
	            		elmView.showErrorMessage("Unable to maximize the window. Please try again.");
					}  
	            	/* setting the internal frame as maximised will automatically invokes the 
					 * internalFrameDeiconified(InternalFrameEvent e) method of the DefaultInternalFrameListener
					 * of the internal frame. This method will make all the other step */
	            } else if(internalFrame.isSelected()) { // if the internal frame is maximised and selected               	                    	
	            	try {
	            		// set the internal frame minimised
	            		internalFrame.setIcon(true);
					} catch (PropertyVetoException exception) {
	            		elmView.showErrorMessage("Unable to minimize the window. Please try again.");
					}                    	
	            	/* setting the internal frame as minimised will automatically invokes the 
					 * internalFrameIconified(InternalFrameEvent e) method of the DefaultInternalFrameListener
					 * of the internal frame. This method will make all the other step */
	            } else { // if the internal frame is maximised but not selected
	            	try {
	            		// set the internal frame selected
	            		internalFrame.setSelected(true);
	        		} catch (PropertyVetoException exception) {
	                	elmView.showErrorMessage("Unable to select the window. Please try again.");
	                } 
	            }
            }
        }
    }     
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the 
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the <code>JMenuItem</code>
     * items of the <code>JPopupMenu</code> pop-up added to an 
     * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton} button.
     * 
     * <p>At the moment it implements the behaviour of the only <code>JMenuItem</code> 
     * item inside the pop-up menu, but it can be modified with other items behaviours in
     * an easy manner if we will add other items to the pop-up menu of the 
     * {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton}
     * button in the future.
     * 
     * <p>In the present case, the only item inside the pop-up menu is the one that expresses
     * the user choice of closing the <code>JInternalFrame</code> frame connected with the
     * button owner of the item.
     * 
     * <p>This listener then will search the corresponding <code>JInternalFrame</code> frame
     * of the {@link unipi.bpm.elm.santeramo.view.FooterAreaButton FooterAreaButton}
     * button, owner of the pressed item, and close it.
     *  
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see ActionListener
     * @see ActionEvent  
     * @see JInternalFrame
     * @see javax.swing.JPopupMenu
     * @see javax.swing.JMenuItem
     */ 
    class PopupMenuItemActionListener implements ActionListener {

    	/**
    	 * Invoked when an action occurs. This method retrieves which item has been pressed
    	 * by checking its action command, composed by two substrings divided by the symbol "-":
    	 * they describe the command to apply (specified by the first string) to a specific  
    	 * <code>JInternalFrame</code> internal frame (specified by the second string).
    	 * 
    	 * <p>Then, it applies the specified command to the specified internal frame.
    	 * 
    	 * @param e the action event generated by a component
    	 * 
    	 * @see JInternalFrame
    	 * @see ActionEvent
    	 */
    	@Override
    	public void actionPerformed(ActionEvent e) {	    		    	
    		/* get the action command of the JMenuItem item that generates the ActionEvent e.
    		 * The action command of the item is composed by two substring divided by the "-" symbol.
    		 * The first part contains the command to apply, the second part the id of the target 
    		 * internal frame */
			String [] parts = e.getActionCommand().split("-", 2);
			String command = parts[0];
			String internalFrameID = parts[1];
			
			switch (command) {
				// if the command is to close the target internal frame
            	case "close":  
            				JInternalFrame internalFrame = findJInternalFrame(internalFrameID);
            				// if we have found the associated internal frame, close it
            				if (internalFrame != null) {
								closeInternalFrame(internalFrame);
            				} else {
            					elmView.showErrorMessage("There is not any document associated with this button.");
            				}
							break;
				// add here other "case: item command"		
            	default:	break;
			}
    	}
    }
    
    /**
     * This inner class implements <code>DocumentListener</code> for creating a specific
     * listener to use on the {@link unipi.bpm.elm.santeramo.view.EventLogEditor EventLogEditor}
     * ({@link unipi.bpm.elm.santeramo.view.EventLogEditor#textArea textArea}).
     * 
     * <p>When an insert or a remove update happens, the listener changes the status of the document
     * owner (an <code>EventLogEditor</code> frame), to show to the user that the document has been
     * modified and to notify the application of this change.
     * 
     * <p>First, it changes its {@link unipi.bpm.elm.santeramo.view.EventLogEditor#isContentModified isContentModified}
     * status, setting it to <code>true</code>. 
     * 
     * <p>Then it sets the visual status of the <code>EventLogEditor</code> in a specific way to show that the
     * content has been modified from the last time (adding an asterisk at the end of the editor title and
     * at the end of the associated <code>FooterAreaButton</code> title).
     * 
     * <p>The listener at this point sets the {@link unipi.bpm.elm.santeramo.view.EventLogEditor#isContentValidated isContentValidated}
     * status of the <code>EventLogEditor</code> to <code>false</code>, this because the content has been modified since last 
     * validation and therefore needs to be validated again.
     * 
     * <p>If there is at least one footprint matrix associated with this editor, the listener resets the list of associated
     * footprint matrices, because they are not valid anymore due to the change just made and need to be recreated.
     * 
     * <p>For the same reason, the listener resets the footprint filter associated to the editor, so the footprint algorithm
     * will be able to generate a new one on next start, and closes the opened filters panel associated to the editor, because
     * it could contain information not valid anymore.
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see DocumentListener
     */
    class EventLogEditorDocumentListener implements DocumentListener {
    	
    	/**
    	 * Gives notification that there was an insert into the document.
    	 * 
    	 * <p>When this happen, this procedure calls the {@link #modifyEditorStatus(DocumentEvent)},
    	 * to modify the status of the document owner to modified and not validated.
    	 * 
    	 * @param e the document event 
    	 * 
    	 * @see DocumentEvent
    	 */
        @Override        
        public void insertUpdate(DocumentEvent e) {
        	modifyEditorStatus(e);
        }
        
        /**
    	 * Gives notification that a portion of the document has been removed.
    	 * 
    	 * <p>When this happen, this procedure calls the {@link #modifyEditorStatus(DocumentEvent)},
    	 * to modify the status of the document owner to modified and not validated.
    	 * 
    	 * @param e the document event 
    	 * 
    	 * @see DocumentEvent
    	 */
        @Override
        public void removeUpdate(DocumentEvent e) {
        	modifyEditorStatus(e);
        }
        
        /**
    	 * Gives notification that an attribute or set of attributes changed.
    	 * 
    	 * @param e the document event 
    	 * 
    	 * @see DocumentEvent
    	 */
        @Override
        public void changedUpdate(DocumentEvent e) { }
        
        /** 
         * This method gets the <code>EventLogEditor</code> owner of the document and
         * changes its status, to show to the user that the document has been modified
         * and to notify the application of this change.
         * 
         * <p>First, it changes its {@link unipi.bpm.elm.santeramo.view.EventLogEditor#isContentModified isContentModified}
         * status, setting it to <code>true</code>. 
         * 
         * <p>Then it sets the visual status of the <code>EventLogEditor</code> in a specific way to show that the
         * content has been modified from the last time (adding an asterisk at the end of the editor title and
         * at the end of the associated <code>FooterAreaButton</code> title).
         * 
         * <p>The method at this point sets the {@link unipi.bpm.elm.santeramo.view.EventLogEditor#isContentValidated isContentValidated}
         * status of the <code>EventLogEditor</code> to <code>false</code>, this because the content has been modified since last 
         * validation and therefore needs to be validated again.
         * 
         * <p>If there is at least one footprint matrix associated with this editor, the method resets the list of associated
         * footprint matrices, because they are not valid anymore due to the change just made and need to be recreated.
         * 
         * <p>For the same reason, the method resets the footprint filter associated to the editor, so the footprint algorithm
         * will be able to generate a new one on next start, and closes the opened filters panel associated to the editor, because
         * it could contain information not valid anymore.
         * 
         * @param e the document event
         * 
         * @see FooterAreaButton
         * @see DocumentEvent
         */ 
     	private void modifyEditorStatus(DocumentEvent e) {
     		EventLogEditor eventLogEditor = (EventLogEditor)e.getDocument().getProperty("owner");
     		if (eventLogEditor != null) {
         		// if the event log editor content is unmodified
         		if (!eventLogEditor.isContentModified()) {
         			// set the editor as modified
         			eventLogEditor.setContentModified(true);
         			// modify its titles adding an asterisk at the end of them
                    String contentModifiedTitle = eventLogEditor.getTitle() + "*";	             
                    modifyJInternalFrameTitles(eventLogEditor, contentModifiedTitle);
         		} 
         		// if the content was validated
         		if (eventLogEditor.isContentValidated()) {
         			// modify validation status to false because we write something new that needs to be validated again
         			eventLogEditor.setContentValidated(false);
         		}
         		// delete the saved data associated with the editor
         		eventLogEditor.resetAll();
     		}
     	}
    }
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the 
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the <code>JButton</code>
     * buttons of the {@link unipi.bpm.elm.santeramo.view.FilterPanel FilterPanel} panel.
     * 
     * The method gets the <code>JButton</code> that generates the event, reads its action 
     * command and implements the linked action.
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see ActionListener
     * @see JButton
     */
    class FilterPanelActionListener implements ActionListener {
    
    	/**
    	 * Invoked when an action occurs. The method gets the <code>JButton</code>
    	 * that generates the event, reads its action command and implements the 
    	 * linked action.
    	 * 
    	 * <p>Specifically, the method gets from the <code>JButton</code> its parent
    	 * component, that is a <code>FilterPanel</code>, reads from this panel the
    	 * values of its form, and depending on the <code>JButton</code> action command,
    	 * it decides how to manage these values and what action to call.
    	 * 
    	 * @param e the action event generated by a component
    	 * 
    	 * @see JButton
    	 * @see ActionEvent
    	 * @see FilterPanel
    	 */
		@Override
		public void actionPerformed(ActionEvent e) {
			// if the event has been generated by a JButton
			if (e.getSource() instanceof JButton) {
				// get the JButton
				JButton selectedButton = (JButton)e.getSource();
				// if the JButton has a "owner" and it is a FilterPanel
				if (selectedButton.getClientProperty("owner") instanceof FilterPanel) {
					// get the FilterPanel, parent of the JButton
					FilterPanel filtersPanel = (FilterPanel)selectedButton.getClientProperty("owner");
					// check is the value of the text field is an Integer
					int minTraceOccurrenceValue;
					try {  
						// if yes, save it and continue
						minTraceOccurrenceValue = Integer.parseInt(filtersPanel.getMinTraceOccurrenceValue()); 								
					} catch(NumberFormatException ex) {  
						// if not, show an error to the user and stop the process
						elmView.showErrorMessage("The minimum number of trace occurrences must be an integer value.");
					    return;
					}
					// if the text field value was and integer, get the editor associated with the panel
					EventLogEditor eventLogEditor = filtersPanel.getEventLogEditor();
					// get the button action command
					String command = e.getActionCommand();		
					// check the action command and decide
					if (command == "set") {	
						// if set, we just need to update the editor filter, using the value of the from inside the FilterPanel
						eventLogEditor.updateFootprintFilter(minTraceOccurrenceValue, filtersPanel.getClassifiers(), filtersPanel.getSelectedClassifier(),
																filtersPanel.getStartEvents(), filtersPanel.getEndEvents());
					} else if (command == "set-start") {
						// if set and start, update the editor filter, using the value of the from inside the FilterPanel
						eventLogEditor.updateFootprintFilter(minTraceOccurrenceValue, filtersPanel.getClassifiers(), filtersPanel.getSelectedClassifier(),
																filtersPanel.getStartEvents(), filtersPanel.getEndEvents());
						// then start the algorithm on the editor
						generateEventLogFootprint(true, eventLogEditor);
					} else if (command == "start") {
						// if start, we create a temp filter, using the value of the from inside the FilterPanel
						FootprintFilter tempFilter = new FootprintFilter();
						tempFilter.update(minTraceOccurrenceValue, filtersPanel.getClassifiers(), filtersPanel.getSelectedClassifier(),
												filtersPanel.getStartEvents(), filtersPanel.getEndEvents());
						// then we start the algorithm on the editor, using the temp filter instead of the editor one
						generateEventLogFootprint(true, eventLogEditor, tempFilter);
					}
				}						
			}
		}			
    }
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the 
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the <code>JButton</code>
     * of all the dialogs that are created to manage the generation of event log from expressions.
     * 
     * <p>The method gets the <code>JButton</code> that generates the event, reads a specific property
     * to get the reference to the connected <code>JTextField</code> where resides the expression 
     * and at the end calls the {@link #generateEventLog(String)} method to implement the linked action.
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see ActionListener
     * @see JButton
     */
    class GenerateEventLogActionListener implements ActionListener {

    	/**
    	 * Invoked when an action occurs. The method gets the <code>JButton</code>
    	 * that generates the event, reads a specific property to get the reference
    	 * to the connected <code>JTextField</code> where resides the expression and
    	 * at the end calls the {@link #generateEventLog(String)} method to implement
    	 * the linked action. 
    	 * 
    	 * @param e the action event generated by a component
    	 * 
    	 * @see JButton
    	 * @see ActionEvent
    	 */
		@Override
		public void actionPerformed(ActionEvent e) {
			// if the event has been generated by a JButton
			if (e.getSource() instanceof JButton) {
				// get the JButton
				JButton selectedButton = (JButton)e.getSource();
				// if the JButton has a "field" and it is a JTextField
				if (selectedButton.getClientProperty("field") instanceof JTextField) {
					// get the text field
					JTextField expField = (JTextField)selectedButton.getClientProperty("field");
					// generate a log from the expression inside the field
					generateEventLog(expField.getText());
				}
			}
		}    	
    }
    
    /**
     * This inner class implements <code>ActionListener</code> for defining the 
     * {@link ActionListener#actionPerformed(ActionEvent) actionPerformed(ActionEvent)}
     * method and creating a specific listener to use on the <code>JButton</code>
     * of the settings panel.
     * 
     * <p>The method gets the <code>JButton</code> that generates the event, reads its action command
     * to know the action to perform and at the end calls the relative method of the Model 
     * to implement the linked action.
     * 
     * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
     * 
     * @see ActionListener
     * @see JButton
     */
    class SettingsActionListener implements ActionListener {
    	@Override
		public void actionPerformed(ActionEvent e) {
			// if the event has been generated by a JButton
			if (e.getSource() instanceof JButton) {
				// get the JButton
				JButton selectedButton = (JButton)e.getSource();
				String actionCommand = selectedButton.getActionCommand();
				switch (actionCommand) {
					// if the user select the browse button
	            	case "browse":  
		    				// if the JButton has a "field" and it is a JTextField
		    				if (selectedButton.getClientProperty("target") instanceof JTextField) {
		    					// get the text field
		    					JTextField target = (JTextField)selectedButton.getClientProperty("target");
		    		        	// get the XSD filter
		    					FileNameExtensionFilter xsdFilter = new FileNameExtensionFilter("XML Schema Definition (*.xsd)", "xsd");
		    		        	// ask the user to select a file
		    					String selectedFile = elmView.showFileSelectionDialog("Select Schema","",xsdFilter);
								// if the user selected a file
		    					if(selectedFile != null) {
		    						// set the target field with the path of the selected file
									target.setText(selectedFile);
		    		        	}
		    				} else {
		    					elmView.showErrorMessage("Unable to set the selected value. Please try again.");
		    				}
							break;
					// if the user select to apply the edits on the XES tab
	            	case "apply-xes":
	            		String applyXesError = "";
	            			// try to get all the four text fields, otherwise stop
	            			if (selectedButton.getClientProperty("xes-url") instanceof JTextField) {
	            				JTextField xesUrl = (JTextField)selectedButton.getClientProperty("xes-url");
	            				if (selectedButton.getClientProperty("xes-local") instanceof JTextField) {
	            					JTextField xesLocal = (JTextField)selectedButton.getClientProperty("xes-local");
	            					if (selectedButton.getClientProperty("xesext-url") instanceof JTextField) {
	            						JTextField xesExtUrl = (JTextField)selectedButton.getClientProperty("xesext-url");
	            						if (selectedButton.getClientProperty("xesext-local") instanceof JTextField) {
	            							JTextField xesExtLocal = (JTextField)selectedButton.getClientProperty("xesext-local");
	            							// if we read all the field, ask the model to set the values inside these fields in the config file
	            							boolean xesUrlFlag = elmModel.setXesSchemaUrl(xesUrl.getText());
	            							boolean xesLocalFlag = elmModel.setXesSchemaLocal(xesLocal.getText());
	            							boolean xesExtUrlFlag = elmModel.setXesExtSchemaUrl(xesExtUrl.getText());
	            							boolean xesExtLocalFlag = elmModel.setXesExtSchemaLocal(xesExtLocal.getText());
	            							// if one of the operation fails
	            							if (!xesUrlFlag || !xesLocalFlag || !xesExtUrlFlag || !xesExtLocalFlag) {
	            								// error
	            								applyXesError = "Missing configuration file. The system is not able to save the new settings.";
	            							}         							
	            						} else {
	            							applyXesError = "Unable to set the XESEXT local path. Please try again.";
	        		    				}
	    	            			} else {
	    	            				applyXesError = "Unable to set the XESEXT URL. Please try again.";
	    		    				}
		            			} else {
		            				applyXesError = "Unable to set the XES local path. Please try again.";
			    				}
	            			} else {
	            				applyXesError = "Unable to set the XES URL. Please try again.";
		    				}
	            			// if the process got an error
	            			if (!applyXesError.isEmpty()) {
	            				// show it to the user
	            				elmView.showErrorMessage(applyXesError);
	            			}
	            			break;
	            	// if the user select to reset all the fields to the respective value of the XES tab
	            	case "reset-xes":	            		
						// retrieve the user's choice, using the message decided before	    			
		    			String titleXes = "Reset";
		    			String messageXes = "The system will restore default values for these fields. \n"
		    							+ "The operation cannot be undone. \n"
		    							+ "Are you sure you want to reset these values? ";
		    			Object[] optionsXes = {"Yes", "   No   "};
						int choiceXes = elmView.showOptionDialog(messageXes, titleXes, JOptionPane.WARNING_MESSAGE, optionsXes, optionsXes[1]);
						// check user's decision
						if (choiceXes == 0) { // if YES, reset
							String resetXesError = "";
							// try to get all the four text fields, otherwise stop
	            			if (selectedButton.getClientProperty("xes-url") instanceof JTextField) {
	            				JTextField xesUrl = (JTextField)selectedButton.getClientProperty("xes-url");
	            				if (selectedButton.getClientProperty("xes-local") instanceof JTextField) {
	            					JTextField xesLocal = (JTextField)selectedButton.getClientProperty("xes-local");
	            					if (selectedButton.getClientProperty("xesext-url") instanceof JTextField) {
	            						JTextField xesExtUrl = (JTextField)selectedButton.getClientProperty("xesext-url");
	            						if (selectedButton.getClientProperty("xesext-local") instanceof JTextField) {
	            							JTextField xesExtLocal = (JTextField)selectedButton.getClientProperty("xesext-local");
	            							// if we have access to all of them, ask the model to reset their values in the config file
	            							boolean resetFlag = elmModel.resetXesConfig();
	            							// if we reset the values
	            							if (resetFlag) {
	            								// set the default values in the target text fields
	                							xesUrl.setText(elmModel.getXesSchemaUrl());
	                							xesLocal.setText(elmModel.getXesSchemaLocal());
	                							xesExtUrl.setText(elmModel.getXesExtSchemaUrl());
	                							xesExtLocal.setText(elmModel.getXesExtSchemaLocal());
	            							} else {
	            								resetXesError = "The system is not able to restore the default settings.";
	            							}
	            						} else {
	            							resetXesError = "Unable to reset the XESEXT local path. Please try again.";
	        		    				}
	    	            			} else {
	    	            				resetXesError = "Unable to reset the XESEXT URL. Please try again.";
	    		    				}
		            			} else {
		            				resetXesError = "Unable to reset the XES local path. Please try again.";
			    				}
	            			} else {
	            				resetXesError = "Unable to reset the XES URL. Please try again.";
		    				}
	            			// if the process got an error
	            			if (!resetXesError.isEmpty()) {
	            				// show it
	            				elmView.showErrorMessage(resetXesError);
	            			}
						}
            			break;
            		// if the user selected to apply the edits in the General tab 
	            	case "apply-elm":
	            		String applyElmError = "";
	            		// try to get all the text fields, otherwise stop
	            			if (selectedButton.getClientProperty("recent-list") instanceof JTextField) {
	            				JTextField recentList = (JTextField)selectedButton.getClientProperty("recent-list");
	            				if (selectedButton.getClientProperty("event-timestamp") instanceof JTextField) {
	            					JTextField eventTimestamp = (JTextField)selectedButton.getClientProperty("event-timestamp");
	            					// if we have access to all of them, apply the edits in the config file using the Model
	            					boolean recentListFlag = elmModel.setRecentListNumber(recentList.getText());
	    							boolean eventTimestampFlag = elmModel.setEventTimestampDistance(eventTimestamp.getText());
	    	            			// if an error occurred			
	    							if (!recentListFlag || !eventTimestampFlag) {
	    								applyElmError = "Missing configuration file. The system is not able to save the new settings.";
	    							} 
		            			} else {
		            				applyElmError = "Unable to set the Event time distance. Please try again.";
			    				}
	            			} else {
	            				applyElmError = "Unable to set the Recent document number. Please try again.";
		    				}
	            			if (!applyElmError.isEmpty()) {
	            				elmView.showErrorMessage(applyElmError);
	            			}
	            			break;
	            	// if the user selects to reset the values of the General Tab
	            	case "reset-elm":	            		
						// retrieve the user's choice, using the message decided before	    			
		    			String titleElm = "Reset";
		    			String messageElm = "The system will restore default values for these fields. \n"
		    							+ "The operation cannot be undone. \n"
		    							+ "Are you sure you want to reset these values? ";
		    			Object[] optionsElm = {"Yes", "   No   "};
						int choiceElm = elmView.showOptionDialog(messageElm, titleElm, JOptionPane.WARNING_MESSAGE, optionsElm, optionsElm[1]);
						// check user's decision
						if (choiceElm == 0) { // if YES, reset
							String resetElmError = "";
							// try to get the text fields, otherwise stop
	            			if (selectedButton.getClientProperty("recent-list") instanceof JTextField) {
	            				JTextField recentList = (JTextField)selectedButton.getClientProperty("recent-list");
	            				if (selectedButton.getClientProperty("event-timestamp") instanceof JTextField) {
	            					JTextField eventTimestamp = (JTextField)selectedButton.getClientProperty("event-timestamp");
	            					// if we have access to all of them, ask the Model to reset their values to default
	            					boolean resetFlag = elmModel.resetElmConfig();
        							// if the reset did not generate an error
        							if (resetFlag) {
        								// set the target fields with the default values
        								recentList.setText(String.valueOf(elmModel.getRecentListNumber()));
        								eventTimestamp.setText(String.valueOf(elmModel.getEventTimestampDistance()));
        							} else {
        								resetElmError = "The system is not able to restore the default settings.";
        							}	            						
	            				} else {
		            				applyElmError = "Unable to reset the Event time distance. Please try again.";
			    				}
	            			} else {
	            				applyElmError = "Unable to reset the Recent document number. Please try again.";
		    				}
	            			// if the process got an error
	            			if (!resetElmError.isEmpty()) {
	            				// show it
	            				elmView.showErrorMessage(resetElmError);
	            			}
						}
            			break;
	            	default:break;
				}

			}
		}  
    }
    
}