/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.controller;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import unipi.bpm.elm.santeramo.model.EventLogManagerModel;
import unipi.bpm.elm.santeramo.view.EventLogManagerView;

/**
 * This class defines the behaviour of all the <code>MenuAreaButton</code> buttons in the
 * application tab menu that are connected to the application in general (information about
 * the system and global settings).
 * 
 * <p>It is used by the Controller that delegates to this class the implementation of the actions
 * connected to these <code>MenuAreaButton</code> buttons.
 * 
 * <p>The Controller receives the user'choice by the relative listener and call the right method
 * of this class.
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 * 
 * @see unipi.bpm.elm.santeramo.view.MenuAreaButton
 * @see EventLogManagerController
 */
public class InfoManager {
	
	/*
	 * THE CONTROLLER, THE VIEW AND MODEL
	 */
	
	/**
	 * The <code>EventLogManagerController</code> element 
	 * that represents the Controller of the application
	 * and that is the one that uses this editor manager.
	 * 
	 * @see EventLogManagerController
	 */
	@SuppressWarnings("unused")
	private EventLogManagerController elmController;	
	
	/**
	 * The <code>EventLogManagerView</code> element 
	 * that represents the View of the application managed
	 * by the Controller. 
	 * 
	 * <p>We set this reference to the View managed by the Controller from the
	 * {@link unipi.bpm.elm.santeramo.controller.EventLogManagerController#getView() getView()}
	 * method of the <code>EventLogManagerController</code>.
	 * 
	 * @see EventLogManagerView
	 */
	private EventLogManagerView elmView;	
	
	/**
	 * The <code>EventLogManagerModel</code> element 
	 * that represents the Model of the application managed
	 * by the Controller. 
	 * 
	 * <p>We set this reference to the Model managed by the Controller from the
	 * {@link unipi.bpm.elm.santeramo.controller.EventLogManagerController#getModel() getModel()}
	 * method of the <code>EventLogManagerController</code>.
	 * 
	 * @see EventLogManagerModel
	 */
	private EventLogManagerModel elmModel;

	
	
	/*
	 * CONSTRUCTOR
	 */
	
	/**
	 * Constructs a new <code>InfoManager</code>, with the <code>EventLogManagerController</code> 
	 * passed as parameter associated to it.
	 * 
	 * <p>The passed Controller is the one that uses this <code>InfoManager</code> class. This class
	 * uses this reference to the Controller to communicate with it, calling some of its methods.
	 * 
	 * <p>This class is delegated by the Controller to implement some operations, for this reason 
	 * it is able to call the View and the Model managed by the Controller directly on behalf of 
	 * the Controller to handle this operation correctly.
	 * 
	 * @param elmController the <code>EventLogManagerController</code> object that represents the Controller and uses this <code>EditorManager</code> 
	 * 
	 * @see EventLogManagerView
	 * @see EventLogManagerController
	 * @see EventLogManagerModel
	 */
	public InfoManager(EventLogManagerController elmController) {	
		this.elmController = elmController;
		// the view to use
		elmView = elmController.getView();
		// the model to use
		elmModel = elmController.getModel();
	}
	
	
	
	/*
	 * METHODS
	 */
	
	/**
	 * This method creates a settings panel where to show the editable settings
	 * and were the user can change them, applying the edits, or resets them,
	 * restoring their default values.
	 * 
	 * <p>Then it asks the View to show it and the passed listener to manage
	 * the user actions.
	 * 
	 * @param listener the <code>ActionListener</code> that will manage the user actions on the panel buttons
	 */
	public void showSettings(ActionListener listener) {
		
		// crate the first tab
		JPanel xesTab = createXesPanel(listener);
		xesTab.setName("xes-tab");
		// create the second tab
		JPanel elmTab = createElmPanel(listener);
		elmTab.setName("elm-tab");
		// create the tabbed menu
		JTabbedPane menu = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);		
		menu.setName("settings-menu");
		// add the tabs to the menu
		menu.addTab("XES", null, xesTab, "XES");
		menu.addTab("General", null, elmTab, "General");
		// ask the View to show the panel
    	elmView.showDialog("Settings", menu, true);
	}
	
	/**
	 * This method defines the structure of the first tab content in the main panel.
	 * 
	 * <p>It will add the passed listener to the panel buttons to manage them.
	 * 
	 * @param listener the <code>ActionListener</code> that will manage the panel buttons
	 * 
	 * @return the created <code>JPanel</code> that represents the content of the first tab of the settings menu
	 */
	private JPanel createXesPanel(ActionListener listener) {
		
		// the panel to set
		JPanel xesPanel = new JPanel();
		// main settings
		xesPanel.setOpaque(true);
		xesPanel.setBorder(BorderFactory.createEmptyBorder(10, 7, 7, 7));
		xesPanel.setLayout(new GridBagLayout());		
		// create the constraints to use for positioning the elements
		GridBagConstraints xesConstr = new GridBagConstraints();
		// set the general constraints
		xesConstr.fill = GridBagConstraints.HORIZONTAL;	
		xesConstr.anchor = GridBagConstraints.WEST; 
		
        // add a separator, setting its position in the layout and a full width dimension
		xesConstr.insets = new Insets(10,5,15,5);
		xesConstr.gridwidth = 4;
		xesConstr.gridx = 0;
		xesConstr.gridy = 0;	
		xesPanel.add(new JSeparator(SwingConstants.HORIZONTAL), xesConstr);
		// restore the column width dimension
		xesConstr.gridwidth = 1;
		xesConstr.insets = new Insets(7,5,7,5);
		
		// put the first label in the panel, setting its position in the layout	
		xesConstr.weightx = 0;
		xesConstr.gridx = 0;
		xesConstr.gridy = 1;				
		xesPanel.add(new JLabel("XES Schema URL:"), xesConstr);
		// put the relative element in the panel, setting its position in the layout		
		xesConstr.weightx = 1;
		xesConstr.gridwidth = 2;
		xesConstr.gridx = 1;
		xesConstr.gridy = 1;
		JTextField xesSchemaURL = new JTextField(40);
		xesSchemaURL.setText(elmModel.getXesSchemaUrl());
		xesPanel.add(xesSchemaURL, xesConstr);
		xesConstr.gridwidth = 1;
		
		// put the second label in the panel, setting its position in the layout
		xesConstr.weightx = 0;
		xesConstr.gridx = 0;
		xesConstr.gridy = 2;				
		xesPanel.add(new JLabel("XES Schema Local Path:"), xesConstr);		
    	// put the relative element in the panel, setting its position in the layout
		xesConstr.weightx = 1;
		xesConstr.gridwidth = 2;
		xesConstr.gridx = 1;
		xesConstr.gridy = 2;
		JTextField xesSchemaPath = new JTextField(40);
	    // set a value inside the text filed, that is the value saved in the filter
		xesSchemaPath.setText(elmModel.getXesSchemaLocal());
		xesPanel.add(xesSchemaPath, xesConstr);
		xesConstr.gridwidth = 1;		
		// put the browse button in the panel, setting its position in the layout
		xesConstr.weightx = 0.2;
		xesConstr.gridx = 3;
		xesConstr.gridy = 2;
		JButton xesBrowseButton = new JButton("Browse..");
		xesBrowseButton.setActionCommand("browse");
		xesBrowseButton.setMargin(new Insets(3,3,3,3));			
		// set the target text field
		xesBrowseButton.putClientProperty("target", xesSchemaPath); 		
		// set the action listener
		xesBrowseButton.addActionListener(listener); 
		xesPanel.add(xesBrowseButton, xesConstr);
		
		// put the third label in the panel, setting its position in the layout
		xesConstr.weightx = 0;
		xesConstr.gridx = 0;
		xesConstr.gridy = 3;				
		xesPanel.add(new JLabel("XESEXT Schema URL:"), xesConstr);    	
    	// put the relative element in the panel, setting its position in the layout
		xesConstr.weightx = 1;
		xesConstr.gridwidth = 2;
		xesConstr.gridx = 1;
		xesConstr.gridy = 3;
		JTextField xesExtSchemaURL = new JTextField(40);
		xesExtSchemaURL.setText(elmModel.getXesExtSchemaUrl());
		xesPanel.add(xesExtSchemaURL, xesConstr);
		xesConstr.gridwidth = 1;
		
		// put the fourth label in the panel, setting its position in the layout
		xesConstr.weightx = 0;
		xesConstr.gridx = 0;
		xesConstr.gridy = 4;				
		xesPanel.add(new JLabel("XESEXT Schema Local Path:"), xesConstr);		
    	// put the relative element in the panel, setting its position in the layout
		xesConstr.weightx = 1;
		xesConstr.gridwidth = 2;
		xesConstr.gridx = 1;
		xesConstr.gridy = 4;
		JTextField xesExtSchemaPath = new JTextField(40);
	    // set a value inside the text filed, that is the value saved in the filter
		xesExtSchemaPath.setText(elmModel.getXesExtSchemaLocal());
		xesPanel.add(xesExtSchemaPath, xesConstr);
		xesConstr.gridwidth = 1;		
		// put the browse button in the panel, setting its position in the layout
		xesConstr.weightx = 0.2;
		xesConstr.gridx = 3;
		xesConstr.gridy = 4;
		JButton xesExtBrowseButton = new JButton("Browse..");
		xesExtBrowseButton.setActionCommand("browse");
		xesExtBrowseButton.setMargin(new Insets(3,3,3,3));		
		// set the target text field
		xesExtBrowseButton.putClientProperty("target", xesExtSchemaPath); 	
		// set the action listener
		xesExtBrowseButton.addActionListener(listener); 
		xesPanel.add(xesExtBrowseButton, xesConstr);
    
        // add a separator, setting its position in the layout and a full width dimension
		xesConstr.insets = new Insets(15,5,10,5);
		xesConstr.gridwidth = 4;
		xesConstr.gridx = 0;
		xesConstr.gridy = 5;	
		xesPanel.add(new JSeparator(SwingConstants.HORIZONTAL), xesConstr);
		// restore the column width dimension
		xesConstr.gridwidth = 1;
		xesConstr.insets = new Insets(7,5,7,5);
		
		// add a whitespace in the last line
		xesConstr.weightx = 1;
		xesConstr.gridx = 1;
		xesConstr.gridy = 6;				
		xesPanel.add(new JLabel(""), xesConstr);
		
		// add the Reset button, setting its position in the layout
		xesConstr.weightx = 0.28;		
		xesConstr.gridx = 2;
		xesConstr.gridy = 6;
		JButton resetSettings = new JButton("Reset");
		resetSettings.setActionCommand("reset-xes");
		resetSettings.setMargin(new Insets(3,3,3,3));	
		// set the field to reset
		resetSettings.putClientProperty("xes-url", xesSchemaURL);			
		resetSettings.putClientProperty("xes-local", xesSchemaPath);
		resetSettings.putClientProperty("xesext-url", xesExtSchemaURL);
		resetSettings.putClientProperty("xesext-local", xesExtSchemaPath);
		// set the action listener
		resetSettings.addActionListener(listener); 
		xesPanel.add(resetSettings, xesConstr);
		
		// put the Apply button in the panel, setting its position in the layout
		xesConstr.weightx = 0.2;
		xesConstr.gridx = 3;
		xesConstr.gridy = 6;
		JButton applySettings = new JButton("Apply");
		applySettings.setActionCommand("apply-xes");
		applySettings.setMargin(new Insets(3,3,3,3));	
		// set the field from where to read the values
		applySettings.putClientProperty("xes-url", xesSchemaURL);			
		applySettings.putClientProperty("xes-local", xesSchemaPath);
		applySettings.putClientProperty("xesext-url", xesExtSchemaURL);
		applySettings.putClientProperty("xesext-local", xesExtSchemaPath);
		// set the action listener
		applySettings.addActionListener(listener); 
		xesPanel.add(applySettings, xesConstr);
		
		return xesPanel;
	}
	
	/**
	 * This method defines the structure of the second tab content in the main panel.
	 * 
	 * <p>It will add the passed listener to the panel buttons to manage them.
	 * 
	 * @param listener the <code>ActionListener</code> that will manage the panel buttons
	 * 
	 * @return the created <code>JPanel</code> that represents the content of the second tab of the settings menu
	 */
	private JPanel createElmPanel(ActionListener listener) {
		
		// the panel to set
		JPanel editorPanel = new JPanel();
		// general settings
		editorPanel.setOpaque(true);
		editorPanel.setBorder(BorderFactory.createEmptyBorder(10, 7, 7, 7));
		editorPanel.setLayout(new GridBagLayout());		
		// create the constraints to use for positioning the elements
		GridBagConstraints editorConstr = new GridBagConstraints();
		// set the general constraints
		editorConstr.fill = GridBagConstraints.HORIZONTAL;			
		editorConstr.anchor = GridBagConstraints.WEST; 
		
        // add a separator, setting its position in the layout and a full width dimension
		editorConstr.insets = new Insets(10,5,15,5);
		editorConstr.gridwidth = 4;
		editorConstr.gridx = 0;
		editorConstr.gridy = 0;	
		editorPanel.add(new JSeparator(SwingConstants.HORIZONTAL), editorConstr);
		// restore the column width dimension
		editorConstr.insets = new Insets(7,5,7,5);
		
		// put the first label in the panel, setting its position in the layout	
		editorConstr.gridwidth = 1;
		editorConstr.weightx = 0;
		editorConstr.gridx = 0;
		editorConstr.gridy = 1;				
		editorPanel.add(new JLabel("Number of recent documents:"), editorConstr);		
		// put the relative element in the panel, setting its position in the layout		
		editorConstr.gridwidth = 2;
		editorConstr.weightx = 1;
		editorConstr.gridx = 1;
		editorConstr.gridy = 1;
		JTextField recentList = new JTextField();
		recentList.setText(String.valueOf(elmModel.getRecentListNumber()));
		editorPanel.add(recentList, editorConstr);
		
		// put the second label in the panel, setting its position in the layout
		editorConstr.gridwidth = 1;
		editorConstr.weightx = 0;
		editorConstr.gridx = 0;
		editorConstr.gridy = 2;				
		editorPanel.add(new JLabel("Event time distance (sec):"), editorConstr);		
    	// put the relative element in the panel, setting its position in the layout
		editorConstr.gridwidth = 2;
		editorConstr.weightx = 1;
		editorConstr.gridx = 1;
		editorConstr.gridy = 2;
		JTextField eventTimestamp = new JTextField();
	    // set a value inside the text filed, that is the value saved in the filter
		eventTimestamp.setText(String.valueOf(elmModel.getEventTimestampDistance()));
		editorPanel.add(eventTimestamp, editorConstr);

    
        // add a separator, setting its position in the layout and a full width dimension
		editorConstr.insets = new Insets(15,5,10,5);
		editorConstr.gridwidth = 4;
		editorConstr.gridx = 0;
		editorConstr.gridy = 3;	
		editorPanel.add(new JSeparator(SwingConstants.HORIZONTAL), editorConstr);
		editorConstr.insets = new Insets(7,5,7,5);
		// restore the column width dimension		
		editorConstr.gridwidth = 1;
		
		// put a whitespace in the last line		
		editorConstr.weightx = 1;
		editorConstr.gridx = 1;
		editorConstr.gridy = 4;				
		editorPanel.add(new JLabel(""), editorConstr);
		
		// put the Reset button in the panel, setting its position in the layout
		editorConstr.weightx = 0.30;		
		editorConstr.gridx = 2;
		editorConstr.gridy = 4;
		JButton resetSettings = new JButton("Reset");
		resetSettings.setActionCommand("reset-elm");
		resetSettings.setMargin(new Insets(3,3,3,3));	
		// set the text fields to reset
		resetSettings.putClientProperty("recent-list", recentList);			
		resetSettings.putClientProperty("event-timestamp", eventTimestamp);
		// set the action listener
		resetSettings.addActionListener(listener); 
		editorPanel.add(resetSettings, editorConstr);
		
		// put the Apply button in the panel, setting its position in the layout
		editorConstr.weightx = 0.30;
		editorConstr.gridx = 3;
		editorConstr.gridy = 4;
		JButton applySettings = new JButton("Apply");
		applySettings.setActionCommand("apply-elm");
		applySettings.setMargin(new Insets(3,3,3,3));	
		// set the text fields from where to read the values
		applySettings.putClientProperty("recent-list", recentList);			
		applySettings.putClientProperty("event-timestamp", eventTimestamp);
		// set the action listener
		applySettings.addActionListener(listener); 
		editorPanel.add(applySettings, editorConstr);
		
		return editorPanel;
	}
	
	/**
	 * This method opens the application credits (About).
	 */
	public void showAbout() {
		// create the panel
		JPanel aboutPanel = new JPanel();
		// set an empty border around the content
		aboutPanel.setBorder(BorderFactory.createEmptyBorder(10, 12, 10, 0));
		// set the panel layout
		aboutPanel.setLayout(new GridBagLayout());
		// create the constraints to use for positioning the elements
		GridBagConstraints constraints = new GridBagConstraints();
		// set the general constraints
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.weightx = 0.5;	
		constraints.gridwidth = 1;
		
		// set and position the left label
		constraints.insets = new Insets(0,5,0,17);
		constraints.gridx = 0;
		constraints.gridy = 0; 		
		String title = "<html><center><b>Event Log Manager</b><br><br>Version: 1.0<br></center></html>";
		JLabel titleLabel = new JLabel(title);
		titleLabel.setFont(new Font("Serif", Font.PLAIN, 15));
		aboutPanel.add(titleLabel, constraints);
		
		// set and position the separator
		constraints.insets = new Insets(0,0,0,12);
		constraints.gridx = 1;
		constraints.gridy = 0; 
		JSeparator separator = new JSeparator(SwingConstants.VERTICAL);
		separator.setPreferredSize(new Dimension(1,190));
		aboutPanel.add(separator, constraints);
		
		// set and position the right label
		constraints.gridx = 2;
		constraints.gridy = 0; 
		String description = "<html><center>"
					+  "<b>University of Pisa</b><br>"
					+  "Master's degree in Business Informatics<br><br>"
					+  "<i>\"Design and development of a system<br>for the management of event logs\"</i><br><br>"
					+  "<b>Project Manager:</b><br>"
					+  "Francesco Santeramo<br>"
					+  "<i>eventlogmanager@francescosanteramo.com</i><br><br>"
					+  "<b>Supervisor:</b><br>"
					+  "Roberto Bruni<br>"
					+  "<i>bruni@di.unipi.it</i>"
					+ "</center></html>";
		JLabel descriptionLabel = new JLabel(description);
		descriptionLabel.setFont(new Font("Verdana", Font.PLAIN, 11));
        aboutPanel.add(descriptionLabel, constraints);
        
        // ask the view to show the about panel inside a modal dialog
		elmView.showDialog("About Event Log Manager", aboutPanel, true);      
	}
	
}
