/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.controller;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.UUID;

import org.deckfour.xes.id.XID;
import org.deckfour.xes.info.XAttributeInfo;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.logging.XLogging;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeID;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.out.XSerializer;
import org.deckfour.xes.util.XsDateTimeConversionJava7;

/**
 * This class implements <code>XSerializer</code> to define a serializer
 * for writing in an output stream in CSV format a given XES model.
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 * 
 * @see XSerializer
 */
public class XesCsvSerializer implements XSerializer{

	/**
	 * Utility object for data conversion, from a date expressed in
	 * some natural form, to a date expressed in the XES standard.
	 * 
	 * @see XsDateTimeConversionJava7
	 */
	XsDateTimeConversionJava7 xsDateTimeConverter = new XsDateTimeConversionJava7();
		
	/**
	 * Returns a brief description of this serialisation.
	 */
	public String getDescription() {
		return "XES CSV Serialization";
	}

	/**
	 * Returns the human-readable name of this serialisation.
	 */
	public String getName() {
		return "XES CSV";
	}

	/**
	 * Returns the name of this serialization's author.
	 */
	public String getAuthor() {
		return "Francesco Santeramo";
	}

	/**
	 * Returns an array of possible file suffices for this serialisation.
	 */
	public String[] getSuffices() {
		return new String[] { "xes" };
	}
	
	/**
	 * Serialises a given log to the given output stream in CSV format.
	 * 
	 * @param log Log to be serialised
	 * @param out OutputStream for serialisation
	 * 
	 * @throws IOException .
	 */
	public void serialize(XLog log, OutputStream out) throws IOException {
		// use the log system to log the serialisation process
		XLogging.log("start serializing log to XES.CSV", XLogging.Importance.DEBUG);
		long start = System.currentTimeMillis();
		// get the system line separator
		String newLine = System.getProperty("line.separator");
		
		// get the log info and the key of all the defined attributes in the log
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);	
		XAttributeInfo attributeInfo = logInfo.getEventAttributeInfo();
		// save the attribute keys in this list
		ArrayList<String> columnNames = new ArrayList<String>(attributeInfo.getAttributeKeys());
		
		// write in the output the first column element
		out.write("trace:id".getBytes());
		// get a string of all the columns names
		StringBuilder columnNameList = new StringBuilder();
		for (String columnName : columnNames) {
			columnNameList.append(",").append(columnName);
		}
		// write in the output all the column names in one line
		if (columnNameList.length() > 0) {				
			out.write(columnNameList.toString().getBytes());
		}	
		out.write(newLine.getBytes());
		
		// list to take trace of all generated XID for traces
		HashSet<XID> xidList = new HashSet<XID>();			
		// for every trace of the log
		for (XTrace trace : log) {
			String traceId = "";
			boolean hasAttribute = false;
			// if the trace has attributes
			if (trace.hasAttributes()) {
				// get them
				XAttributeMap traceMap = trace.getAttributes();	
				// check if there is a trace id
				if (traceMap.containsKey("identity:id")) {	
					// if the trace has an id, get its value
					XAttribute attribute = traceMap.get("identity:id");
					if (attribute instanceof XAttributeID) {															
						traceId = attribute.toString();
						hasAttribute = true;
					}													
				}
			}
			// if the trace did not have an id as attribute
			if (!hasAttribute) {
				// get an unique id for the trace inside the log 
				XID xid = new XID(UUID.randomUUID());
				// if this XID has already been generated for a trace in this log
        		while (xidList.contains(xid)) {
        			// get another XID and try to check again
        			xid = new XID(UUID.randomUUID());
        		}
        		// add the unique XID to the xidList
        		xidList.add(xid);
        		// get the string representation of the generated xid
				traceId = xid.toString();
			}									
			// for every event in the trace
			for (XEvent event : trace) {
				// if the event has attributes
				if (event.hasAttributes()) {
					// get them
					XAttributeMap attributeMap = event.getAttributes();
					// start recording in a single line the values for this event
					StringBuilder attributeValues = new StringBuilder();
					// first add the trace id of this event
					attributeValues.append(traceId);
					// for every column name in the first line
					for (String columnName : columnNames) {
						// add the CSV separator
						attributeValues.append(",");
						// if the event attribute map contains the attribute defined by the column name
						if (attributeMap.containsKey(columnName)) {										
							// get it
							XAttribute attribute = attributeMap.get(columnName);
							String value = null;	
							// depending on the attribute type, read its value
							if (attribute instanceof XAttributeLiteral || attribute instanceof XAttributeDiscrete ||
								attribute instanceof XAttributeContinuous || attribute instanceof XAttributeBoolean ||
								attribute instanceof XAttributeID) {															
								value = attribute.toString();
							} else if (attribute instanceof XAttributeTimestamp) {
								Date timestamp = ((XAttributeTimestamp)attribute).getValue();									
								value = xsDateTimeConverter.format(timestamp);
							} else {
								value = "___ERROR___";
							}	
							// add the read value to the string line and go to the next column name
							attributeValues.append(value);
						}
						/* if the event does not have this attribute, the process will go to the next column
						 * leaving an empty value for this one (i.e "value1,value2,,value4", as CSV standard) */
					}
					// write the just created line of values in the output and go to the next event
					out.write(attributeValues.toString().getBytes());
					out.write(newLine.getBytes());
				}
			}
		}
		// calculate the duration of the process and print it to the log system
		String duration = " (" + (System.currentTimeMillis() - start) + " msec.)";
		XLogging.log("finished serializing log" + duration, XLogging.Importance.DEBUG);
	}

	/**
	 * Returns the value of the {@link #getName()} method.
	 */
	public String toString() {
		return this.getName();
	}
}
