/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.view;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.DefaultDesktopManager;
import javax.swing.JInternalFrame;

/**
 * This class extends <code>DefaultDesktopManager</code> to create a new DesktopManger able to manage the position 
 * of every new <code>JInternalFrame</code> frame positioned inside a <code>JDesktopPane</code> (in our case 
 * represented by the {@link unipi.bpm.elm.santeramo.view.EventLogManagerView#contentArea contentArea} area).
 * 
 * <p>When we want to find the best position for a new <code>JInternalFrame</code> frame inside a <code>JDesktopPane</code> area,
 * we can ask to the <code>CustomDesktopManager</code> of that area to find it, calling its {@link #positionFrame(JInternalFrame)} 
 * method and passing to it the internal frame to position.
 *  
 * <p>This method first tries to put the frame in the first empty space in the diagonal that goes from NW to SE,
 * either at the end of the pile of the already opened frames or in one space between them left free
 * from a frame that has been closed or moved out from its original position. 
 * 
 * <p>If the method is not able to find any free space on this way, due to the end of free space 
 * on the main window, it positions the internal frame on the centre of the area, over 
 * all the other internal frames. 
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 * 
 * @see DefaultDesktopManager
 * @see JInternalFrame
 * @see javax.swing.JDesktopPane
 */
public class CustomDesktopManager extends DefaultDesktopManager {
	
	/*
	 * VARIABLES
	 */
	
	/**
	 * Auto-Generated Serial Version UID for this class.
	 */
	private static final long serialVersionUID = 2621536282324506115L;
	
	
	
	/*
	 * CONSTRUCTOR
	 */
	
	/**
	 * Constructor for this class. It invokes the super class constructor.
	 */
	public CustomDesktopManager() {
		super();
	}	
	
	
	
	/*
	 * METHODS
	 */
	
	/** 
	 * This method tries to find the best position for a <code>JInternalFrame</code>
	 * inside its<code>JDesktopPane</code> and to set its location if there is any free
	 * space on the area (otherwise the internal frame get positioned on the default
	 * position by the application, over all the other frames).
	 * 
	 * @param f the internal frame to position
	 * 
	 * @see JInternalFrame
	 * @see javax.swing.JDesktopPane
	 */
	public void positionFrame(JInternalFrame f) {
		JInternalFrame targetInternalFrame = f;
		/* if the position of this component is 0,0 it means that this component doesn't have a position yet on the screen
		 * and we need to find one for it (or it means that this is the component that resides in position 0,0 but in that
		 * case this method will simply reposition it there). If it has a position already, it means that this component
		 * is an old one that has been minimised and now gets maximised, so we don't need to calculate a new position for
		 * it and it will just get back to its position when maximised.*/
		if ((targetInternalFrame.getX() == 0) && (targetInternalFrame.getY() == 0)) {    	
    		// get all the components (Internal Frames) in the container (JDesktop Pane)
			JInternalFrame[] allInternalFrames = null;
			synchronized (targetInternalFrame.getDesktopPane().getTreeLock()) {
				allInternalFrames = targetInternalFrame.getDesktopPane().getAllFrames();
			}
    		// starting point where the research begins
    		Point startingPoint = new Point(0,0);
    		// size of the container (JDesktop Pane) where to research free space
    		Dimension containerSize = targetInternalFrame.getDesktopPane().getSize();
    		// point where to place the component at the end
    		Point position = null;
    		
    		// try to find the point
    		position = firstFreePosition(startingPoint, containerSize, allInternalFrames, targetInternalFrame);
    		// if we found a good position, set the location of the component
    		if (position != null) {
    			targetInternalFrame.setLocation(position.x,position.y);
    		}
    		/* if we didn't find a point, the component get positioned 
    		* by the application in a default position on the screen */
		}
	}
	
	/**
	 * This method tries to find a place, defined by a NW corner point,
	 * in the container area, defined by its size, where there are no other 
	 * internal frames and there is enough space to place the target internal frame.
	 * 
	 * <p>Starting from a given starting point, this method checks if there is already an
	 * internal frame in that position. If that position is free, it returns that point as
	 * the first free one found. Otherwise, if there is already an internal frame in that
	 * position, the method checks a new point defined by an offset added to the last 
	 * checked point.
	 * 
	 * <p>The method continues on this way until either it finds a free point to return
	 * or it reaches the bounds of the container and so there is no more space to check.
	 * 
	 * @param startingPoint starting point where to start searching
	 * @param containerSize size of the container where to research free space
	 * @param allInternalFrames all the internal frames in the container
	 * @param targetInternalFrame the target internal frame to position
	 * 
	 * @return the first free point found where to place the internal frame or null if none has been found within the bounds.
	 */
	private Point firstFreePosition(Point startingPoint, Dimension containerSize, JInternalFrame[] allInternalFrames, JInternalFrame targetInternalFrame) {    		
		// the space is free?
		boolean isFree = false;
		// point that represents at the end of the process the first free position found, starting from the starting point
		Point freePosition = new Point(startingPoint);
		// offset to use when searching a new free position, when the last checked was occupied
		int offset = 20;
    
		/* until we don't find a free space, continue to search (we exit from this loop when we find a 
		 * free space or with an inside return statement in case we finish the space where to search) */
		while (!isFree) {	
			// we suppose to find a free space at the actual value of the variable freePosition and we set to true the value of isFree 
			isFree = true;
			// then we check if our hypothesis is right, looking if there is already an internal frame in that position or not
			for (int i = 0; i < allInternalFrames.length; i++) { //for every component in the container				
				// if the internal frame that we are checking is the same that we try to position, skip to the next one
				if (allInternalFrames[i].equals(targetInternalFrame)) continue;
				// otherwise, get the position of the internal frame that we are checking now
				Rectangle checkedPosition = allInternalFrames[i].getBounds();
				// check if the freePosition that I suppose to be free, is instead occupied by this internal frame
				if (checkedPosition.x == freePosition.x && checkedPosition.y == freePosition.y) {
					// if this internal frame is in the position that we supposed empty, then we need to check a new position
					freePosition.x = freePosition.x + offset;
					freePosition.y = freePosition.y + offset;
					// if the new position that we want to check as next is too close to the borders of the main window
					if ((freePosition.x + offset) > containerSize.width || (freePosition.y + offset) > containerSize.height) {
						return null;  // it is not possible to find an empty position for this internal frame
					}
					// we were wrong at the beginning on our supposition, this position is not empty, so we set isFree as false
					isFree = false;
					// we don't need to check the remaining internal frames, because we already found one in the position we were looking for
					break;
				}
			}
		}
		// if no internal frames have been found in the freePosition we were looking for, then isFree has remained true and we have found an empty position
		return freePosition;
	}
	
}
