/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.view;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;

/**
 * This class extends <code>JToggleButton</code> for defining a specific type of button. 
 *  
 * <p>A new <code>FooterAreaButton</code> button gets positioned on the 
 * {@link EventLogManagerView#footerArea footerArea}
 * from the View whenever a new <code>JInternalFrame</code> frame gets added
 * to the {@link EventLogManagerView#contentArea contentArea}.
 * 
 * <p>This class defines specific graphic settings and appearance for this
 * type of button and makes possible to specify a listener for it and for
 * the item of its <code>JPopupMenu</code> pop-up menu.
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 * 
 * @see JToggleButton
 * @see javax.swing.JInternalFrame
 * @see JPopupMenu
 */
public class FooterAreaButton extends JToggleButton {
	
	/*
	 * VARIABLES
	 */
	
	/**
	 * Auto-Generated Serial Version UID for this class.
	 */
	private static final long serialVersionUID = -2356474387379017537L;

	
	/*
	 * CONSTRUCTORS
	 */

	/**
	 * Creates an <code>FooterAreaButton</code> button with the specified text and 
	 * selection state, calling the constructor of the super class and then the method 
	 * {@link FooterAreaButton#buttonSettings(String, int)} to set all the button details,
	 * included the identifier of the <code>JInternalFrame</code> to manage.
	 * 
	 * @param text the text of the button
	 * @param selected if true, the button is initially selected; otherwise, the button is initially unselected
	 * @param internalFrameID the identifier of the <code>JInternalFrame</code> to manage
	 * 
	 * @see javax.swing.JInternalFrame
	 */
	public FooterAreaButton(String text, boolean selected, int internalFrameID) {
		super(text, selected);	
		buttonSettings(text, internalFrameID);
	}
	
	/**
	 * Creates a <code>FooterAreaButton</code> button with the specified text, image, and selection state,
	 * calling the constructor of the super class and then the method {@link FooterAreaButton#buttonSettings(String, int)} 
	 * to set all the button details, included the identifier of the <code>JInternalFrame</code> to manage.
	 * 
	 * @param text the text of the button
	 * @param icon the Icon image to display on the button
	 * @param selected if true, the button is initially selected; otherwise, the button is initially unselected
	 * @param internalFrameID the identifier of the <code>JInternalFrame</code> to manage
	 */
	public FooterAreaButton(String text, Icon icon, boolean selected, int internalFrameID) {
		super(text, icon, selected);
		buttonSettings(text, internalFrameID);			
	}
	
	
	/*
	 * METHODS
	 */
	
	/**
	 * Sets the graphic details and settings of the button, included its pop-up.
	 * 
	 * @param text the text of the button
	 * @param internalFrameID the identifier of the <code>JInternalFrame</code> to manage
	 */
	private void buttonSettings(String text, int internalFrameID) {		
		//set name using the id of the internal frame to manage
		this.setName(String.valueOf(internalFrameID));
		// set size
		this.setPreferredSize(new Dimension(getPreferredSize().width + 20, getPreferredSize().height));	
		this.setMinimumSize(this.getPreferredSize());
		this.setMaximumSize(this.getPreferredSize());
		// set tool tip
		this.setToolTipText(text);  
			
		// create a pop-up menu to associate to a right click on this button
		JPopupMenu rightClickPopup = new JPopupMenu();
		// create a JMenuItem item with text and an icon if the icon is available, with only text otherwise
		JMenuItem closeItem;   
		String resourcePath = "/images/close-little.png";
        URL iconURLClose = this.getClass().getResource(resourcePath);                        
        if (iconURLClose != null) {            	
        	closeItem = new JMenuItem("Close", new ImageIcon(Toolkit.getDefaultToolkit().getImage(iconURLClose)));
        } else {            	
        	closeItem = new JMenuItem("Close");
        }	
        // set the action command of the item
        closeItem.setActionCommand("close-" + String.valueOf(internalFrameID));
        // set the tool tip of the item
        closeItem.setToolTipText("Close");
        // add the item to the pop-up menu
        rightClickPopup.add(closeItem);
        // set rightClickPopup as the default pop-up of the button
		this.setComponentPopupMenu(rightClickPopup);
	}
	
	/**
	 * Returns the preferred location to display the pop-up menu in this component's coordinate system. 
	 * 
	 * <p>In this case, the preferred location is on the upper right side of the button that own the pop-up.
	 * 
	 * @param event the MouseEvent that triggered the pop-up to be shown, or null if the pop-up is not being shown as the result of a mouse event 
	 *
	 * @return location to display the JPopupMenu, null otherwise.
	 */
	@Override
    public Point getPopupLocation(MouseEvent event) {		
        // get the registered pop-up menu
        JPopupMenu popupMenu = getComponentPopupMenu();
        // get the super point
        Point popupPosition = super.getPopupLocation(event);
        
        if (popupMenu != null) {           
            // create a new pop-up position and set x and y coordinates
        	popupPosition = new Point();
        	popupPosition.x = (getWidth()-45);
        	popupPosition.y = -(getHeight());
        }
        return popupPosition;
    }
	
	/**
	 * Sets an <code>ActionListener</code> to the pop-up menu items of the button.
	 * 
	 * @param listener the <code>ActionListener</code> to add to the pop-up menu items
	 */
	public void addPopupMenuItemsListener(ActionListener listener) {			
		// get the pop-up menu associated with this button
		JPopupMenu popupMenu = this.getComponentPopupMenu();
		Component[] popupMenuItems;
		synchronized (popupMenu.getTreeLock()) {
			// get all its components
			popupMenuItems = popupMenu.getComponents();
		}		
		// for every component of the pop-up menu
		for (int i = 0; i < popupMenuItems.length; i++) {
			// if the component is an item of the pop-up
			if (popupMenuItems[i] instanceof JMenuItem) {
				// add the passed listener to the item
				((JMenuItem)popupMenuItems[i]).addActionListener(listener);		
			}		
		}
	}
	
	
}  