/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.view;

import java.util.Collection;
import java.util.HashMap;

import org.deckfour.xes.classification.XEventClassifier;

/**
 * This class defines a filter, composed by different saved selections,
 * to use when analysing the content of a specific <code>EventLogEditor</code>.
 * 
 * <p>Every <code>EventLogEditor</code> has an object of this class that
 * implements its personal filter. When the content of the editor
 * gets analysed, this filter is used during analysis.
 * 
 * <p>This class records the user's choices on the {@link FilterPanel} and
 * save them as default filter selections for its editor owner.
 * 
 * The user can modify the <code>EventLogEditor</code> filter selections
 * using the {@link FilterPanel}, shown by the View for this purpose.
 * 
 * The filter can save this user's selections to give them back to the analysis
 * process: the minimum number of occurrence for a trace to be considered for the analysis,
 * the list of registered classifiers for the <code>EventLogEditor</code> content, the selected
 * classifier from the list to be used for the analysis, the possible start events and end events
 * and the ones selected to be considered during the analysis (if a trace starts or ends with an
 * event not included in this selection, the trace gets discarded).
 * 
 * @see EventLogEditor
 * 
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 */
public class FootprintFilter {

	/*
	 * VARIABLES
	 */
	
	/**
	 * This variable shows the state of the filter.
	 * 
	 * <p>If the filter has been updated with all the requested 
	 * information, the value will be <code>true</code>. At the 
	 * beginning, when created, since the filter gets updated
	 * or until it has incomplete information, the value is
	 * <code>false</code>.
	 */
	private boolean complete;
	
	/**
	 * The minimum number of occurrences that a trace needs to
	 * have to be considered during the analysis.
	 */
	private int minTraceOccurrences;
	
	/**
	 * The <code>Collection</code> of <code>XEventClassifier</code>
	 * representing the list of registered classifiers for the 
	 * <code>EventLogEditor</code> content.
	 * 
	 * @see XEventClassifier
	 * @see EventLogEditor
	 */
	private Collection<XEventClassifier> classifiers;
	
	/**
	 * The <code>XEventClassifier</code> selected for the analysis.
	 * 
	 * @see XEventClassifier
	 */
	private XEventClassifier selectedClassifier;
	
	/**
	 * This map contains all the possible start events (recorded by
	 * name) for the considered log and an associated boolean value
	 * that represents if the start event has been selected or not
	 * for the analysis.
	 */
	private HashMap<String, Boolean> startEvents;
	
	/**
	 * This map contains all the possible end events (recorded by
	 * name) for the considered log and an associated boolean value
	 * that represents if the end event has been selected or not
	 * for the analysis.
	 */
	private HashMap<String, Boolean> endEvents;
	
	
	
	/*
	 * CONSTRUCTOR
	 */
	
	/**
	 * Creates a <code>FootprintFilter</code> with default values
	 * (an incomplete filter with the value of {@link #minTraceOccurrences}
	 * set as 1 and all the others as <code>null</code>).
	 */
	public FootprintFilter() {
		// filter incomplete
		complete = false;
		// minimum number of occurrences
		minTraceOccurrences = 1;
		// all the other values are null because we cannot know them before the analysis
		classifiers = null;
		selectedClassifier = null;
		startEvents = null;
		endEvents = null;
	}

	
	
	/*
	 * METHODS
	 */
	
	/**
	 * Updates the filter variables with the passed parameters.
	 * 
	 * <p>If one or more passed values are <code>null</code>, the filter
	 * will be set as incomplete, otherwise if all the passed parameters have
	 * valid values, the filter will be set as complete.
	 * 
	 * @param minTraceOccurrences the minimum number of occurrences for a trace to be considered during the analysis
	 * @param classifiers the <code>Collection</code> of <code>XEventClassifier</code> representing the list of registered classifiers for the considered log
	 * @param selectedClassifier the <code>XEventClassifier</code> selected for the log analysis
	 * @param startEvents the <code>HashMap</code> containing all the possible start events (recorded by name) for the considered log
	 * 						 and an associated boolean value that represents if the start event has been selected or not for the analysis
	 * @param endEvents the <code>HashMap</code> containing all the possible end events (recorded by name) for the considered log
	 * 						 and an associated boolean value that represents if the end event has been selected or not for the analysis
	 */
	public void update(int minTraceOccurrences, Collection<XEventClassifier> classifiers, XEventClassifier selectedClassifier, 
								HashMap<String, Boolean> startEvents, HashMap<String, Boolean> endEvents) {
		// try to set the filter as complete	
		setComplete(true);
		// then call the set methods to set all the values	
		setMinTraceOccurrences(minTraceOccurrences);			
		setClassifiers(classifiers);
		setSelectedClassifier(selectedClassifier);			
		setStartEvents(startEvents);			
		setEndEvents(endEvents);
		// if just one of the values is null, the respective set method will set the filter as incomplete
	}
	
	/**
	 * Resets the filter, keeping the {@link #minTraceOccurrences} value
	 * and removing all the others.
	 * 
	 * This method restores the filter to its incomplete status, without
	 * the calculated values from the analysis.
	 */
	public void reset() {
		update(getMinTraceOccurrences(), null, null, null, null);
	}

	/**
	 * Returns <code>true</code> if the filter is {@link #complete},
	 * <code>false</code> otherwise.
	 * 
	 * @return <code>true</code> if the filter is complete, <code>false</code> otherwise.
	 */
	public boolean isComplete() {
		return complete;		
	}
	
	/**
	 * Sets the filter as complete or incomplete, depending on the value the passed parameter,
	 * modifying the value of the {@link #complete} variable.
	 * 
	 * @param complete <code>true</code> to set the filter as complete, <code>false</code> otherwise
	 */
	private void setComplete(boolean complete) {
		this.complete = complete;
	}
	
	/**
	 * Returns the minimum number of occurrence for a trace to be considered during the analysis.
	 * 
	 * @return the minimum number of occurrence for a trace to be considered during the analysis.
	 */
	public int getMinTraceOccurrences() {
		return minTraceOccurrences;
	}
	
	/**
	 * Sets the minimum number of occurrence for a trace to be considered during the analysis.
	 * 
	 * <p>If the passed value is lower than 1, the method use a value equal to 1.
	 * 
	 * @param minTraceOccurrences the minimum number of occurrence for a trace to be considered during the analysis
	 */
	private void setMinTraceOccurrences(int minTraceOccurrences) {
		if (minTraceOccurrences > 0) {
			this.minTraceOccurrences = minTraceOccurrences;
		} else {
			this.minTraceOccurrences = 1;
		}			
	}
	
	/**
	 * Returns the <code>Collection</code> of the registered <code>XEventClassifier</code> for the associated log.
	 * 
	 * @return the <code>Collection</code> of the registered <code>XEventClassifier</code> for the considered log.
	 */
	public Collection<XEventClassifier> getClassifiers() {
		return classifiers;
	}
	
	
	/**
	 * Sets the <code>Collection</code> of the registered <code>XEventClassifier</code> for the associated log.
	 * 
	 * <p>If the passed value is <code>null</code>, this method sets the filter as not {@link #complete}.
	 * 
	 * @param classifiers the <code>Collection</code> of the registered <code>XEventClassifier</code> for the considered log
	 */
	private void setClassifiers(Collection<XEventClassifier> classifiers) {
		this.classifiers = classifiers;
		// if the passed value is null, set the filter as incomplete
		if (classifiers == null) {
			setComplete(false);
		}
	}
	
	/**
	 * Returns the <code>XEventClassifier</code> selected for the log analysis.
	 * 
	 * @return the <code>XEventClassifier</code> selected for the log analysis.
	 */
	public XEventClassifier getSelectedClassifier() {
		return selectedClassifier;
	}
	
	
	/**
	 * Sets the <code>XEventClassifier</code> to use for the log analysis.
	 * 
	 * <p>If the passed value is <code>null</code>, this method sets the filter as not {@link #complete}.
	 * 
	 * @param selectedClassifier the <code>XEventClassifier</code> to use for the log analysis
	 */
	private void setSelectedClassifier(XEventClassifier selectedClassifier) {
		this.selectedClassifier = selectedClassifier;
		// if the passed value is null, set the filter as incomplete
		if (selectedClassifier == null) {
			setComplete(false);
		}
	}
	
	/**
	 * Returns the <code>HashMap</code> containing all the possible start events 
	 * (recorded by name) for the considered log with an associated boolean value that represents
	 * if the start event has been selected or not for the analysis.
	 * 
	 * @return the <code>HashMap</code> containing all the possible start events (recorded by name) and an associated boolean value that represents the possible selection.
	 */
	public HashMap<String, Boolean> getStartEvents() {
		return startEvents;
	}
	
	/**
	 * Sets the <code>HashMap</code> containing all the possible start events 
	 * (recorded by name) for the considered log with an associated boolean value that represents
	 * if the start event has been selected or not for the analysis.
	 * 
	 * <p>If the passed value is <code>null</code>, this method sets the filter as not {@link #complete}.
	 * 
	 * @param startEvents the <code>HashMap</code> containing all the possible start events (recorded by name) and an associated boolean value that represents the possible selection
	 */
	private void setStartEvents(HashMap<String, Boolean> startEvents) {
		this.startEvents = startEvents;
		// if the passed value is null, set the filter as incomplete
		if (startEvents == null) {
			setComplete(false);
		}
	}
	
	/**
	 * Returns the <code>HashMap</code> containing all the possible end events 
	 * (recorded by name) for the considered log with an associated boolean value that represents
	 * if the end event has been selected or not for the analysis.
	 * 
	 * @return the <code>HashMap</code> containing all the possible end events (recorded by name) and an associated boolean value that represents the possible selection.
	 */
	public HashMap<String, Boolean> getEndEvents() {
		return endEvents;
	}
	
	/**
	 * Sets the <code>HashMap</code> containing all the possible end events 
	 * (recorded by name) for the considered log with an associated boolean value that represents
	 * if the end event has been selected or not for the analysis.
	 * 
	 * <p>If the passed value is <code>null</code>, this method sets the filter as not {@link #complete}.
	 * 
	 * @param endEvents the <code>HashMap</code> containing all the possible end events (recorded by name) and an associated boolean value that represents the possible selection
	 */
	private void setEndEvents(HashMap<String, Boolean> endEvents) {
		this.endEvents = endEvents;
		// if the passed value is null, set the filter as incomplete
		if (endEvents == null) {
			setComplete(false);
		}
	}
}
