/*
 * Event Log Manager
 *   
 *  Copyright (C) 2015 Francesco Santeramo
 *
 *  This file is part of Event Log Manager.
 *
 *  Event Log Manager is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Event Log Manager is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *	
 *  Author:  Francesco Santeramo 
 *  Website: www.francescosanteramo.com
 *  Contact: eventlogmanager@francescosanteramo.com    
 */

package unipi.bpm.elm.santeramo.view;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;

/**
 * This class extends <code>FlowLayout</code> to create a particular version
 * of this type of layout.
 * 
 * <p>In details, this class implements a layout that acts as the <code>FlowLayout</code>
 * layout until there is enough space in the container to position all the components
 * inside one single row. 
 * 
 * <p>When there is not enough space for positioning all the components in one single line
 * inside the container, differently from the <code>FlowLayout</code> layout that starts putting
 * the remaining components on a new line and continues on this way until all of them are
 * positioned, the <code>OneRowFlowLayout</code> layout resizes the components to maintain them
 * all in one single row.
 *  
 * @author Francesco Santeramo (eventlogmanager@francescosanteramo.com)
 * 
 * @see FlowLayout
 */
public class OneRowFlowLayout extends FlowLayout {
	
	/*
	 * VARIABLES
	 */
	
	/**
	 * Auto-Generated Serial Version UID for this class.
	 */
	private static final long serialVersionUID = -6327683884666003524L;

	
	
	/*
	 * CONSTRUCTORS
	 */
	
	/**
	 * Constructor with no parameters that calls the {@link #OneRowFlowLayout(int, int, int)} 
	 * constructor to create a layout with default parameters (align=LEFT, hgap=0, vgap=0).
	 */
	public OneRowFlowLayout() {
      this(FlowLayout.LEFT, 0, 0);
	}
	
	/**
	 * Constructor with alignment parameter that calls the {@link #OneRowFlowLayout(int, int, int)}
	 * constructor to create a layout with the specified alignment and default hgap and vgap 
	 * (hgap=0, vgap=0).
	 * 
	 * @param align alignment of the positioned components
	 */
	public OneRowFlowLayout(int align) {
	    this(align, 0, 0);
	}
		
	/**
	 * Constructor with alignment, horizontal gap and vertical gap parameters that calls
	 * the super class constructor to create a layout with the specified information.
	 *  
	 * @param align alignment of the positioned components
	 * @param hgap horizontal gap between components
	 * @param vgap vertical gap between components
	 */
	public OneRowFlowLayout(int align, int hgap, int vgap) {
	    super(align, hgap, vgap);
	}
	
	
	
	/*
	 * METHODS
	 */

	/**
     * Lays out the container. This method lets each visible component take its
     * preferred size until there is enough space for displaying all the components
     * in one single row inside the container.
     * 
     * <p>When there is not enough space for positioning all the components with their
     * preferred size in one single row inside the container, the method finds the
     * maximum possible component size for displaying all the components in one single
     * row; then it positions every component using its preferred size if smaller than
     * the maximum possible one, the maximum possible size otherwise.
     *
     * @param target the specified container being laid out
     */
	@Override
	public void layoutContainer(Container target) {	 
		
		synchronized (target.getTreeLock()) {  
			// get the number of components inside the container
			int componentsNumber = target.getComponentCount();
			// if there are no components inside the container, there is nothing to do
			if (componentsNumber == 0) {
				return;
			}
			// get the insets of the container
			Insets containerInsets = target.getInsets();
			// get the width and height of the container without the insets			
			int containerWidth = target.getWidth() - (containerInsets.left + containerInsets.right + getHgap()*2);
			int containerHeight = target.getHeight() - (containerInsets.top + containerInsets.bottom + getVgap()*2);
			/* get the maximum possible component width by taking the container width, subtracting the horizontal
			 * gap of every components inside of it except the last one that is not used (because it does not
			 * have another component after it) and dividing this value for the number of the components.
			 */
			int componentWidth = (containerWidth - (componentsNumber - 1) * getHgap()) / componentsNumber;		
			
			// set the x and y values of the NW corner point of the position of the first component
			int y = containerInsets.top + getVgap();			
			int x = containerInsets.left + getHgap();

			// for every component of the container
			for (int i = 0; i < componentsNumber; i++) {	
				// get the component
				Component component = target.getComponent(i);
				if (component.isVisible()) {
					// define the maximum size that this component can have
					Dimension maxPossibleSize = new Dimension(componentWidth, containerHeight);
					// get the preferred size of the component
					Dimension wantedSize = component.getPreferredSize();				
					/* select the minimum of the wanted size and the max possible size.
					 * Thus, if the wanted size is smaller than the max possible size that
					 * this component can have, we set the component with its wanted size;
					 * otherwise if the component has a wanted size bigger than the maximum
					 * possible size, we shrink the component to its max possible dimension */				
					wantedSize = selectSize(wantedSize,maxPossibleSize);
					// set the new position and dimension of the component
					component.setBounds(x, y, wantedSize.width, wantedSize.height);
					// update the x value for defining the position of the next component to insert inside the container
					x = x + wantedSize.width + getHgap();
					component.revalidate();
					component.repaint();
				}
			}
		}
	}
	
	/**
	 * This method returns a new dimension built with the smallest width and height
	 * of the two dimensions passed as parameters.
	 * 
	 * @param wantedSize the dimension wanted by the component
	 * @param maxPossibleSize the maximum possible dimension for the component
	 * 
	 * @return a new dimension with the smallest width and height of the two passed dimensions.
	 */
	private Dimension selectSize(Dimension wantedSize, Dimension maxPossibleSize) {
		
		int width = 0;
		int height = 0;
		// if the width of the wanted size is littler, take that, otherwise use the max possible size width
		if (wantedSize.width < maxPossibleSize.width) {
			width = wantedSize.width;
		} else {
			width = maxPossibleSize.width;
		}
		// if the height of the wanted size is littler, take that, otherwise use the max possible size height
		if (wantedSize.height < maxPossibleSize.height) {
			height = wantedSize.height;
		} else {
			height = maxPossibleSize.height;
		}
		// return a new dimension for the component built with the smallest width and height
		return new Dimension(width, height);
	}
}
